# -*- coding: utf-8 -*-
# filename: test_async_agent_e2e.py
# @Time    : 2025/10/05 15:47
# @Author  : JQQ
# @Email   : jqq1716@gmail.com
# @Software: PyCharm
"""
中文: 异步 Agent 模块的端到端测试：基于真实 HTTP 服务与真实 AsyncSMCPAgentClient，验证核心功能流。
English: End-to-end tests for the Async Agent module: real HTTP service and real AsyncSMCPAgentClient validating core functionality flows.
"""

from __future__ import annotations

import asyncio

import pytest

from a2c_smcp.agent import AsyncSMCPAgentClient, DefaultAgentAuthProvider
from a2c_smcp.smcp import (
    JOIN_OFFICE_EVENT,
    LEAVE_OFFICE_EVENT,
    SMCP_NAMESPACE,
    EnterOfficeNotification,
    LeaveOfficeNotification,
    SMCPTool,
    UpdateMCPConfigNotification,
)

pytestmark = pytest.mark.e2e


async def _wait_until(cond, timeout: float = 2.0, step: float = 0.01) -> bool:
    """
    中文: 简易异步等待辅助函数，直到条件满足或超时。
    English: Simple async wait helper until condition met or timeout.
    """
    end = asyncio.get_event_loop().time() + timeout
    while asyncio.get_event_loop().time() < end:
        if cond():
            return True
        await asyncio.sleep(step)
    return cond()


class MockAsyncEventHandler:
    """
    中文: 测试用的异步事件处理器，记录所有事件
    English: Test async event handler that records all events
    """

    def __init__(self):
        self.enter_office_events: list[EnterOfficeNotification] = []
        self.leave_office_events: list[LeaveOfficeNotification] = []
        self.update_config_events: list[UpdateMCPConfigNotification] = []
        self.tools_received_events: list[tuple[str, list[SMCPTool]]] = []

    async def on_computer_enter_office(self, data: EnterOfficeNotification, sio: AsyncSMCPAgentClient) -> None:
        self.enter_office_events.append(data)

    async def on_computer_leave_office(self, data: LeaveOfficeNotification, sio: AsyncSMCPAgentClient) -> None:
        self.leave_office_events.append(data)

    async def on_computer_update_config(self, data: UpdateMCPConfigNotification, sio: AsyncSMCPAgentClient) -> None:
        self.update_config_events.append(data)

    async def on_tools_received(self, computer: str, tools: list[SMCPTool], sio: AsyncSMCPAgentClient) -> None:
        self.tools_received_events.append((computer, tools))


@pytest.mark.asyncio
async def test_async_agent_connect_and_join_office(async_socketio_server, async_server_port: int, async_mock_computer_client):
    """
    中文:
      - 验证异步 Agent 客户端连接到服务器
      - 验证异步 Agent 加入办公室
      - 验证 Computer 先加入后 Agent 能收到通知
    English:
      - Verify async Agent client connects to server
      - Verify async Agent joins office
      - Verify Agent receives notification when Computer joins first
    """
    # 创建认证提供者 / Create auth provider
    auth_provider = DefaultAgentAuthProvider(
        agent_id="test-agent-async-1",
        office_id="office-async-1",
    )

    # 创建事件处理器 / Create event handler
    event_handler = MockAsyncEventHandler()

    # 创建异步 Agent 客户端 / Create async Agent client
    agent_client = AsyncSMCPAgentClient(
        auth_provider=auth_provider,
        event_handler=event_handler,
    )

    try:
        # 连接到服务器 / Connect to server
        await agent_client.connect_to_server(f"http://127.0.0.1:{async_server_port}")

        # 等待连接稳定 / Wait for connection to stabilize
        await asyncio.sleep(0.1)

        # Agent 加入办公室 / Agent joins office
        ok, err = await agent_client.call(
            JOIN_OFFICE_EVENT,
            {"role": "agent", "name": "test-agent-async-1", "office_id": "office-async-1"},
            namespace=SMCP_NAMESPACE,
            timeout=5,
        )
        assert ok is True
        assert err is None

        # Computer 加入办公室 / Computer joins office
        ok, err = await async_mock_computer_client.call(
            JOIN_OFFICE_EVENT,
            {"role": "computer", "name": "test-computer-async-1", "office_id": "office-async-1"},
            namespace=SMCP_NAMESPACE,
            timeout=5,
        )
        assert ok is True
        assert err is None

        # 等待事件处理 / Wait for event processing
        assert await _wait_until(lambda: len(event_handler.enter_office_events) >= 1, timeout=3)

        # 验证收到的事件 / Verify received events
        assert len(event_handler.enter_office_events) >= 1
        enter_event = event_handler.enter_office_events[0]
        assert enter_event["office_id"] == "office-async-1"
        assert "computer" in enter_event

        # 验证自动获取工具列表 / Verify automatic tools fetching
        assert await _wait_until(lambda: len(event_handler.tools_received_events) >= 1, timeout=3)
        computer_id, tools = event_handler.tools_received_events[0]
        assert len(tools) == 2  # echo and add
        assert tools[0]["name"] == "echo"
        assert tools[1]["name"] == "add"

    finally:
        # 清理连接 / Cleanup connection
        await agent_client.disconnect()


@pytest.mark.asyncio
async def test_async_agent_tool_call(async_socketio_server, async_server_port: int, async_mock_computer_client):
    """
    中文:
      - 验证异步 Agent 调用 Computer 工具
      - 验证工具调用返回正确结果
    English:
      - Verify async Agent calls Computer tools
      - Verify tool call returns correct results
    """
    auth_provider = DefaultAgentAuthProvider(
        agent_id="test-agent-async-2",
        office_id="office-async-2",
    )

    agent_client = AsyncSMCPAgentClient(
        auth_provider=auth_provider,
        event_handler=None,
    )

    try:
        # 连接并加入办公室 / Connect and join office
        await agent_client.connect_to_server(f"http://127.0.0.1:{async_server_port}")
        await asyncio.sleep(0.1)

        await agent_client.call(
            JOIN_OFFICE_EVENT,
            {"role": "agent", "name": "test-agent-async-2", "office_id": "office-async-2"},
            namespace=SMCP_NAMESPACE,
            timeout=5,
        )

        # Computer 加入办公室 / Computer joins office
        await async_mock_computer_client.call(
            JOIN_OFFICE_EVENT,
            {"role": "computer", "name": "test-computer-async-2", "office_id": "office-async-2"},
            namespace=SMCP_NAMESPACE,
            timeout=5,
        )

        # 等待连接稳定 / Wait for connection to stabilize
        await asyncio.sleep(0.2)

        # 调用 echo 工具 / Call echo tool
        result = await agent_client.emit_tool_call(
            computer="test-computer-async-2",
            tool_name="echo",
            params={"message": "Hello, Async World!"},
            timeout=5,
        )

        # 验证结果 / Verify result
        assert result.isError is False
        assert len(result.content) == 1
        assert "Echo: Hello, Async World!" in result.content[0].text

        # 调用 add 工具 / Call add tool
        result = await agent_client.emit_tool_call(
            computer="test-computer-async-2",
            tool_name="add",
            params={"a": 15, "b": 25},
            timeout=5,
        )

        # 验证结果 / Verify result
        assert result.isError is False
        assert len(result.content) == 1
        assert "Result: 40" in result.content[0].text

    finally:
        await agent_client.disconnect()


@pytest.mark.asyncio
async def test_async_agent_get_tools_and_desktop(async_socketio_server, async_server_port: int, async_mock_computer_client):
    """
    中文:
      - 验证异步 Agent 获取 Computer 工具列表
      - 验证异步 Agent 获取 Computer 桌面信息
    English:
      - Verify async Agent gets Computer tools list
      - Verify async Agent gets Computer desktop info
    """
    auth_provider = DefaultAgentAuthProvider(
        agent_id="test-agent-async-3",
        office_id="office-async-3",
    )

    agent_client = AsyncSMCPAgentClient(
        auth_provider=auth_provider,
        event_handler=None,
    )

    try:
        # 连接并加入办公室 / Connect and join office
        await agent_client.connect_to_server(f"http://127.0.0.1:{async_server_port}")
        await asyncio.sleep(0.1)

        await agent_client.call(
            JOIN_OFFICE_EVENT,
            {"role": "agent", "name": "test-agent-async-3", "office_id": "office-async-3"},
            namespace=SMCP_NAMESPACE,
            timeout=5,
        )

        # Computer 加入办公室 / Computer joins office
        await async_mock_computer_client.call(
            JOIN_OFFICE_EVENT,
            {"role": "computer", "name": "test-computer-async-3", "office_id": "office-async-3"},
            namespace=SMCP_NAMESPACE,
            timeout=5,
        )

        await asyncio.sleep(0.2)

        # 获取工具列表 / Get tools list
        tools_response = await agent_client.get_tools_from_computer("test-computer-async-3", timeout=5)

        # 验证工具列表 / Verify tools list
        assert tools_response["req_id"] is not None
        assert len(tools_response["tools"]) == 2
        assert tools_response["tools"][0]["name"] == "echo"
        assert tools_response["tools"][1]["name"] == "add"

        # 获取桌面信息 / Get desktop info
        desktop_response = await agent_client.get_desktop_from_computer("test-computer-async-3", timeout=5)

        # 验证桌面信息 / Verify desktop info
        assert desktop_response["req_id"] is not None
        assert len(desktop_response["desktops"]) == 2
        assert "window://1" in desktop_response["desktops"]
        assert "window://2" in desktop_response["desktops"]

    finally:
        await agent_client.disconnect()


@pytest.mark.asyncio
async def test_async_agent_computer_leave_notification(async_socketio_server, async_server_port: int, async_mock_computer_client):
    """
    中文:
      - 验证 Computer 离开办公室时异步 Agent 收到通知
    English:
      - Verify async Agent receives notification when Computer leaves office
    """
    auth_provider = DefaultAgentAuthProvider(
        agent_id="test-agent-async-4",
        office_id="office-async-4",
    )

    event_handler = MockAsyncEventHandler()

    agent_client = AsyncSMCPAgentClient(
        auth_provider=auth_provider,
        event_handler=event_handler,
    )

    try:
        # 连接并加入办公室 / Connect and join office
        await agent_client.connect_to_server(f"http://127.0.0.1:{async_server_port}")
        await asyncio.sleep(0.1)

        await agent_client.call(
            JOIN_OFFICE_EVENT,
            {"role": "agent", "name": "test-agent-async-4", "office_id": "office-async-4"},
            namespace=SMCP_NAMESPACE,
            timeout=5,
        )

        # Computer 加入办公室 / Computer joins office
        await async_mock_computer_client.call(
            JOIN_OFFICE_EVENT,
            {"role": "computer", "name": "test-computer-async-4", "office_id": "office-async-4"},
            namespace=SMCP_NAMESPACE,
            timeout=5,
        )

        # 等待加入事件和工具获取完成 / Wait for join event and tools fetching
        assert await _wait_until(lambda: len(event_handler.enter_office_events) >= 1, timeout=3)
        # 等待工具列表自动获取完成 / Wait for automatic tools fetching to complete
        assert await _wait_until(lambda: len(event_handler.tools_received_events) >= 1, timeout=3)

        # Computer 离开办公室 / Computer leaves office
        await async_mock_computer_client.call(
            LEAVE_OFFICE_EVENT,
            {"office_id": "office-async-4"},
            namespace=SMCP_NAMESPACE,
            timeout=5,
        )

        # 等待离开事件 / Wait for leave event
        assert await _wait_until(lambda: len(event_handler.leave_office_events) >= 1, timeout=3)

        # 验证离开事件 / Verify leave event
        leave_event = event_handler.leave_office_events[0]
        assert leave_event["office_id"] == "office-async-4"
        assert "computer" in leave_event

    finally:
        await agent_client.disconnect()


@pytest.mark.asyncio
async def test_async_agent_multiple_tool_calls(async_socketio_server, async_server_port: int, async_mock_computer_client):
    """
    中文:
      - 验证异步 Agent 可以连续调用多个工具
      - 验证并发工具调用的正确性
    English:
      - Verify async Agent can call multiple tools consecutively
      - Verify correctness of concurrent tool calls
    """
    auth_provider = DefaultAgentAuthProvider(
        agent_id="test-agent-async-5",
        office_id="office-async-5",
    )

    agent_client = AsyncSMCPAgentClient(
        auth_provider=auth_provider,
        event_handler=None,
    )

    try:
        # 连接并加入办公室 / Connect and join office
        await agent_client.connect_to_server(f"http://127.0.0.1:{async_server_port}")
        await asyncio.sleep(0.1)

        await agent_client.call(
            JOIN_OFFICE_EVENT,
            {"role": "agent", "name": "test-agent-async-5", "office_id": "office-async-5"},
            namespace=SMCP_NAMESPACE,
            timeout=5,
        )

        # Computer 加入办公室 / Computer joins office
        await async_mock_computer_client.call(
            JOIN_OFFICE_EVENT,
            {"role": "computer", "name": "test-computer-async-5", "office_id": "office-async-5"},
            namespace=SMCP_NAMESPACE,
            timeout=5,
        )

        await asyncio.sleep(0.2)

        # 连续调用多个工具 / Call multiple tools consecutively
        results = []
        for i in range(3):
            result = await agent_client.emit_tool_call(
                computer="test-computer-async-5",
                tool_name="add",
                params={"a": i, "b": i + 1},
                timeout=5,
            )
            results.append(result)

        # 验证所有调用都成功 / Verify all calls succeeded
        assert len(results) == 3
        for i, result in enumerate(results):
            assert result.isError is False
            expected = i + (i + 1)
            assert f"Result: {expected}" in result.content[0].text

    finally:
        await agent_client.disconnect()


@pytest.mark.asyncio
async def test_async_agent_list_room(async_socketio_server, async_server_port: int, async_mock_computer_client):
    """
    中文:
      - 验证异步 Agent 调用 LIST_ROOM 事件
      - 验证返回房间内所有会话信息（Agent 和 Computer）
      - 验证会话信息包含正确的 sid、name、role、office_id
    English:
      - Verify async Agent calls LIST_ROOM event
      - Verify returns all session info in the room (Agent and Computer)
      - Verify session info contains correct sid, name, role, office_id
    """
    from a2c_smcp.smcp import LIST_ROOM_EVENT

    auth_provider = DefaultAgentAuthProvider(
        agent_id="test-agent-async-list-room",
        office_id="office-list-room-e2e",
    )

    agent_client = AsyncSMCPAgentClient(
        auth_provider=auth_provider,
        event_handler=None,
    )

    try:
        # 连接并加入办公室 / Connect and join office
        await agent_client.connect_to_server(f"http://127.0.0.1:{async_server_port}")
        await asyncio.sleep(0.1)

        office_id = "office-list-room-e2e"
        await agent_client.call(
            JOIN_OFFICE_EVENT,
            {"role": "agent", "name": "agent-list-test", "office_id": office_id},
            namespace=SMCP_NAMESPACE,
            timeout=5,
        )

        # Computer 加入办公室 / Computer joins office
        await async_mock_computer_client.call(
            JOIN_OFFICE_EVENT,
            {"role": "computer", "name": "computer-list-test", "office_id": office_id},
            namespace=SMCP_NAMESPACE,
            timeout=5,
        )

        # 等待连接稳定 / Wait for connection to stabilize
        await asyncio.sleep(0.2)

        # 获取 Agent 和 Computer 的 SID / Get Agent and Computer SID
        agent_sid = agent_client.get_sid(namespace=SMCP_NAMESPACE)
        computer_sid = async_mock_computer_client.get_sid(namespace=SMCP_NAMESPACE)

        # 调用 LIST_ROOM 事件 / Call LIST_ROOM event
        result = await agent_client.call(
            LIST_ROOM_EVENT,
            {
                "agent": agent_sid,
                "req_id": "list_room_req_e2e",
                "office_id": office_id,
            },
            namespace=SMCP_NAMESPACE,
            timeout=5,
        )

        # 验证返回结果 / Verify result
        assert result is not None
        assert "sessions" in result
        assert "req_id" in result
        assert result["req_id"] == "list_room_req_e2e"

        # 验证会话列表 / Verify session list
        sessions = result["sessions"]
        assert len(sessions) == 2  # 1 Agent + 1 Computer

        # 提取角色列表 / Extract role list
        roles = [s["role"] for s in sessions]
        assert "agent" in roles
        assert "computer" in roles

        # 验证 Agent 会话信息 / Verify Agent session info
        agent_session = next(s for s in sessions if s["role"] == "agent")
        assert agent_session["sid"] == agent_sid
        assert agent_session["name"] == "agent-list-test"
        assert agent_session["office_id"] == office_id

        # 验证 Computer 会话信息 / Verify Computer session info
        computer_session = next(s for s in sessions if s["role"] == "computer")
        assert computer_session["sid"] == computer_sid
        assert computer_session["office_id"] == office_id

    finally:
        await agent_client.disconnect()



"""
* 文件名: types
* 作者: JQQ
* 创建日期: 2025/9/29
* 最后修改日期: 2025/9/29
* 版权: 2023 JQQ. All rights reserved.
* 依赖: None
* 描述: Server端类型定义 / Server-side type definitions
"""

from typing import Literal, TypeAlias

from typing_extensions import TypedDict

# 类型别名定义 / Type aliases
OFFICE_ID: TypeAlias = str
SID: TypeAlias = str


class BaseSession(TypedDict):
    """
    公共会话基类，包含sid和name属性
    Base session class, includes sid and name attributes
    """

    sid: str  # 会话ID / Session ID
    name: str  # 会话名称 / Session name


class ComputerSession(BaseSession):
    """
    Computer会话类型
    Computer session type
    """

    role: Literal["computer"]
    office_id: str


class AgentSession(BaseSession):
    """
    Agent会话类型
    Agent session type
    """

    role: Literal["agent"]
    office_id: str


# 联合类型定义 / Union type definition
Session = ComputerSession | AgentSession



# -*- coding: utf-8 -*-
# filename: test_stdio_client.py
# @Time    : 2025/8/19 17:00
# @Author  : JQQ
# @Email   : jqq1716@gmail.com
# @Software: PyCharm
# filename: test_std_mcp_client.py
import sys
from collections.abc import Callable
from pathlib import Path

import pytest
from mcp import StdioServerParameters
from mcp.types import (
    CallToolResult,
    PromptListChangedNotification,
    ResourceListChangedNotification,
    ServerNotification,
    ToolListChangedNotification,
)
from pydantic import BaseModel
from transitions import MachineError

from a2c_smcp.computer.mcp_clients.stdio_client import StdioMCPClient


@pytest.mark.asyncio
async def test_state_transitions(
    stdio_params: StdioServerParameters,
    track_state: tuple[Callable[[str, str], None], list[tuple[str, str]]],
) -> None:
    """测试客户端状态转换 Test client state transitions"""
    callback, history = track_state

    client: StdioMCPClient = StdioMCPClient(stdio_params, state_change_callback=callback)

    # 初始状态检查 Initial state check
    assert client.state == "initialized"

    # 连接服务器 Connect to server
    await client.aconnect()
    assert client.state == "connected"
    assert ("initialized", "connected") in history

    # 断开连接 Disconnect
    await client.adisconnect()
    assert client.state == "disconnected"
    assert ("connected", "disconnected") in history

    # 重新初始化 Re-initialize
    await client.ainitialize()
    assert client.state == "initialized"
    assert ("disconnected", "initialized") in history


@pytest.mark.asyncio
async def test_list_tools(stdio_params: StdioServerParameters) -> None:
    """测试获取工具列表功能 Test getting tool list"""
    client: StdioMCPClient = StdioMCPClient(stdio_params)
    await client.aconnect()

    tools = await client.list_tools()

    # 验证工具列表 Validate tool list
    assert len(tools) == 1
    assert tools[0].name == "hello"
    assert tools[0].description == "Say hello to someone."

    # 清理客户端 Cleanup client
    await client.adisconnect()


@pytest.mark.asyncio
async def test_call_tool_success(stdio_params: StdioServerParameters) -> None:
    """测试成功调用工具 Test successful tool call"""
    client: StdioMCPClient = StdioMCPClient(stdio_params)
    await client.aconnect()

    # 使用默认参数调用 Call with default params
    result_default: CallToolResult = await client.call_tool("hello", {})
    assert isinstance(result_default, CallToolResult)
    assert not result_default.isError
    assert result_default.content[0].text == "Hello, World!"

    # 使用自定义参数调用 Call with custom params
    result_custom: CallToolResult = await client.call_tool("hello", {"name": "Alice"})
    assert not result_custom.isError
    assert result_custom.content[0].text == "Hello, Alice!"

    # 清理客户端 Cleanup client
    await client.adisconnect()


@pytest.mark.asyncio
async def test_call_tool_failure(stdio_params: StdioServerParameters) -> None:
    """测试工具调用失败场景 Test tool call failure"""
    client: StdioMCPClient = StdioMCPClient(stdio_params)
    await client.aconnect()

    # 调用不存在的工具 Call non-existent tool
    result: CallToolResult = await client.call_tool("nonexistent_tool", {})
    assert result.isError, "调用不存在的工具应该失败 Calling non-existent tool should fail"

    # 调用工具但参数错误 Call tool with wrong params
    result = await client.call_tool("hello", {"invalid_param": "value"})
    assert isinstance(result, CallToolResult)
    # 虽然参数错误，但由于mcp server有一定的容错能力，可以返回成功 Even with wrong params, mcp server may return success
    assert not result.isError
    assert result.content[0].text == "Hello, World!"

    # 清理客户端 Cleanup client
    await client.adisconnect()


@pytest.mark.asyncio
async def test_async_session_property(stdio_params: StdioServerParameters) -> None:
    """测试 async_session 属性 Test async_session property"""
    client: StdioMCPClient = StdioMCPClient(stdio_params)

    # 未连接状态下会话为空 Session is None when not connected
    assert client._async_session is None

    # 在未连接状态下访问 @async_property async_session 会触发自动连接，因此会话不为空 Accessing async_session triggers auto-connect
    assert (await client.async_session) is not None

    with pytest.raises(MachineError) as e:
        # 在连接状态下访问 Access when connected
        await client.aconnect()

    assert "Can't trigger event aconnect from state connected!" in str(e.value)
    session = await client.async_session
    assert session is not None

    # 清理客户端 Cleanup client
    await client.adisconnect()


@pytest.mark.asyncio
async def test_invalid_state_operations(stdio_params: StdioServerParameters) -> None:
    """测试在无效状态下执行操作 Test operations in invalid state"""
    client: StdioMCPClient = StdioMCPClient(stdio_params)

    # 在未连接状态下调用工具 Call tool when not connected
    with pytest.raises(ConnectionError):
        await client.call_tool("hello", {})

    # 在未连接状态下获取工具列表 Get tool list when not connected
    with pytest.raises(ConnectionError):
        await client.list_tools()

    # 尝试在未连接状态下断开 Try disconnect when not connected
    with pytest.raises(MachineError) as e:
        await client.adisconnect()
    assert "Can't trigger event adisconnect from state initialized!" in str(e.value)


@pytest.mark.asyncio
async def test_error_recovery(
    stdio_params: StdioServerParameters,
    track_state: tuple[Callable[[str, str], None], list[tuple[str, str]]],
) -> None:
    """测试错误状态恢复 Test error recovery"""
    callback, history = track_state

    client: StdioMCPClient = StdioMCPClient(stdio_params, state_change_callback=callback)
    await client.aconnect()

    # 强制进入错误状态 Force error state
    await client.aerror()
    assert client.state == "error"
    assert any(from_state == "connected" and to_state == "error" for from_state, to_state in history)

    # 从错误状态恢复 Recover from error
    await client.ainitialize()
    assert client.state == "initialized"
    assert ("error", "initialized") in history

    # 尝试重新连接 Try reconnect
    await client.aconnect()
    assert client.state == "connected"

    # 清理 Cleanup
    await client.adisconnect()


@pytest.mark.asyncio
async def test_stdio_tools_list_and_call():
    """
    中文: 连接带工具的服务器，列出工具并调用 echo。
    英文: Connect to server with tools, list tools and call echo.
    """
    server_py = Path(__file__).resolve().parents[2] / "computer" / "mcp_servers" / "tool_stdio_server.py"
    assert server_py.exists(), f"server script not found: {server_py}"

    params = StdioServerParameters(command=sys.executable, args=[str(server_py)])

    client = StdioMCPClient(params)

    # 连接 / connect
    await client.aconnect()
    await client._create_session_success_event.wait()

    # 初始化结果检查 / initialize_result
    assert client.initialize_result is not None
    assert client.initialize_result.serverInfo.name == "tool-itest-server"
    assert client.initialize_result.capabilities.tools, "tools配置非空"
    assert not client.initialize_result.capabilities.tools.listChanged, "工具列表未开始变化通知"

    # 获取工具列表 / list tools
    tools = await client.list_tools()
    names = {t.name for t in tools}
    assert "echo" in names

    # 调用工具 / call tool
    result = await client.call_tool("echo", {"text": "hello"})
    # CallToolResult.content 是 list[ContentBlock]，期望第一条为 TextContent("echo: hello")
    assert result.content and getattr(result.content[0], "type", None) == "text"
    assert getattr(result.content[0], "text", None) == "echo: hello"

    # 断开 / disconnect
    await client.adisconnect()
    await client._async_session_closed_event.wait()
    assert client.initialize_result is None


class DummyParams(BaseModel):
    pass


@pytest.mark.asyncio
async def test_stdio_initialize_result_end_to_end():
    """
    中文: 通过真实stdio子进程建立连接，校验 initialize_result 的赋值与清理。
    英文: Establish real stdio connection via subprocess and validate initialize_result set and cleared.
    """
    # 服务器脚本路径 / server script path
    server_py = Path(__file__).resolve().parents[2] / "computer" / "mcp_servers" / "minimal_stdio_server.py"
    assert server_py.exists(), f"server script not found: {server_py}"

    # 构建 StdioServerParameters，使用当前解释器运行脚本 / use current interpreter to run script
    params = StdioServerParameters(command=sys.executable, args=[str(server_py)])

    client = StdioMCPClient(params)

    # 连接 / connect
    await client.aconnect()
    await client._create_session_success_event.wait()

    # 验证初始化结果存在且包含服务器信息 / ensure initialize_result present with server info
    assert client.initialize_result is not None
    assert client.initialize_result.serverInfo.name == "itest-server"

    # 断开 / disconnect
    await client.adisconnect()
    await client._async_session_closed_event.wait()

    # 验证清理 / ensure cleared
    assert client.initialize_result is None


@pytest.mark.asyncio
async def test_stdio_message_handler_receives_list_changed_notifications() -> None:
    """
    中文: 验证在初始化 StdioMCPClient 时传入 message_handler，能接收 listChanged 通知。
    英文: Verify that passing message_handler to StdioMCPClient captures listChanged notifications.
    """
    server_py = Path(__file__).resolve().parents[2] / "computer" / "mcp_servers" / "notifications_stdio_server.py"
    assert server_py.exists(), f"server script not found: {server_py}"

    params = StdioServerParameters(command=sys.executable, args=[str(server_py)])

    received = {"tools": 0, "resources": 0, "prompts": 0}

    async def message_handler(message):
        from mcp.types import (
            PromptListChangedNotification,
            ResourceListChangedNotification,
            ServerNotification,
            ToolListChangedNotification,
        )

        if isinstance(message, ServerNotification):
            if isinstance(message.root, ToolListChangedNotification):
                received["tools"] += 1
            elif isinstance(message.root, ResourceListChangedNotification):
                received["resources"] += 1
            elif isinstance(message.root, PromptListChangedNotification):
                received["prompts"] += 1

    client = StdioMCPClient(params, message_handler=message_handler)

    await client.aconnect()
    await client._create_session_success_event.wait()

    # 触发通知 / trigger notifications
    result = await client.call_tool("trigger_changes", {})
    assert result.content and getattr(result.content[0], "type", None) == "text"

    import anyio

    # 等待异步消息处理 / wait up to ~2s
    for _ in range(20):
        if received["tools"] >= 1 and received["resources"] >= 1 and received["prompts"] >= 1:
            break
        await anyio.sleep(0.1)

    assert received["tools"] >= 1
    assert received["resources"] >= 1
    assert received["prompts"] >= 1

    await client.adisconnect()



"""
* 文件名: test_auth
* 作者: JQQ
* 创建日期: 2025/9/30
* 最后修改日期: 2025/9/30
* 版权: 2023 JQQ. All rights reserved.
* 依赖: pytest
* 描述: Agent认证模块测试用例 / Agent authentication module test cases
"""

import pytest

from a2c_smcp.agent.auth import AgentAuthProvider, DefaultAgentAuthProvider
from a2c_smcp.agent.types import AgentConfig


class TestAgentAuthProvider:
    """测试Agent认证提供者抽象基类 / Test Agent authentication provider abstract base class"""

    def test_abstract_methods(self) -> None:
        """测试抽象方法不能直接实例化 / Test abstract methods cannot be instantiated directly"""
        with pytest.raises(TypeError):
            AgentAuthProvider()  # type: ignore


class TestDefaultAgentAuthProvider:
    """测试默认Agent认证提供者 / Test default Agent authentication provider"""

    def test_init_with_minimal_params(self) -> None:
        """测试使用最小参数初始化 / Test initialization with minimal parameters"""
        provider = DefaultAgentAuthProvider(
            agent_id="test_agent",
            office_id="test_office",
        )

        assert provider.get_agent_id() == "test_agent"

        config = provider.get_agent_config()
        assert config["agent"] == "test_agent"
        assert config["office_id"] == "test_office"

    def test_init_with_full_params(self) -> None:
        """测试使用完整参数初始化 / Test initialization with full parameters"""
        provider = DefaultAgentAuthProvider(
            agent_id="test_agent",
            office_id="test_office",
            api_key="test_key",
            api_key_header="Authorization",
            extra_headers={"Custom-Header": "custom_value"},
            auth_data={"token": "test_token"},
        )

        assert provider.get_agent_id() == "test_agent"

        # 测试认证数据
        # Test authentication data
        auth_data = provider.get_connection_auth()
        assert auth_data == {"token": "test_token"}

        # 测试请求头
        # Test headers
        headers = provider.get_connection_headers()
        assert headers["Authorization"] == "test_key"
        assert headers["Custom-Header"] == "custom_value"

    def test_get_connection_auth_empty(self) -> None:
        """测试空认证数据 / Test empty authentication data"""
        provider = DefaultAgentAuthProvider(
            agent_id="test_agent",
            office_id="test_office",
        )

        auth_data = provider.get_connection_auth()
        assert auth_data is None

    def test_get_connection_headers_with_api_key(self) -> None:
        """测试带API密钥的请求头 / Test headers with API key"""
        provider = DefaultAgentAuthProvider(
            agent_id="test_agent",
            office_id="test_office",
            api_key="secret_key",
        )

        headers = provider.get_connection_headers()
        assert headers["x-api-key"] == "secret_key"

    def test_get_connection_headers_without_api_key(self) -> None:
        """测试不带API密钥的请求头 / Test headers without API key"""
        provider = DefaultAgentAuthProvider(
            agent_id="test_agent",
            office_id="test_office",
        )

        headers = provider.get_connection_headers()
        assert "x-api-key" not in headers

    def test_get_agent_config(self) -> None:
        """测试获取Agent配置 / Test get Agent configuration"""
        provider = DefaultAgentAuthProvider(
            agent_id="test_agent_123",
            office_id="test_office_456",
        )

        config = provider.get_agent_config()
        expected_config: AgentConfig = {
            "agent": "test_agent_123",
            "office_id": "test_office_456",
        }
        assert config == expected_config

    def test_custom_api_key_header(self) -> None:
        """测试自定义API密钥请求头 / Test custom API key header"""
        provider = DefaultAgentAuthProvider(
            agent_id="test_agent",
            office_id="test_office",
            api_key="test_key",
            api_key_header="X-Custom-Auth",
        )

        headers = provider.get_connection_headers()
        assert headers["X-Custom-Auth"] == "test_key"
        assert "x-api-key" not in headers



# -*- coding: utf-8 -*-
"""
文件名: test_utils_more.py
作者: JQQ
创建日期: 2025/9/29
最后修改日期: 2025/9/29
版权: 2023 JQQ. All rights reserved.
依赖: pytest
描述:
  中文: 补充覆盖 `print_mcp_config` 的空与非空场景，确保稳定输出。
  English: Add coverage for `print_mcp_config` empty and non-empty cases to ensure stable printing.
"""

from __future__ import annotations

from a2c_smcp.computer.cli.utils import print_mcp_config


def test_print_mcp_config_empty() -> None:
    # 不应抛出异常 / Should not raise
    print_mcp_config({})


def test_print_mcp_config_non_empty() -> None:
    cfg = {
        "servers": {
            "s1": {"type": "stdio", "disabled": False},
            "s2": {"type": "http", "disabled": True},
        },
        "inputs": [
            {"id": "A", "type": "promptString", "description": "alpha"},
            {"id": "B", "type": "pickString", "description": "beta"},
        ],
    }
    print_mcp_config(cfg)



# -*- coding: utf-8 -*-
"""
测试 a2c_smcp/server/base.py
"""

from __future__ import annotations

from unittest.mock import AsyncMock, MagicMock

import pytest

from a2c_smcp.server.auth import AuthenticationProvider
from a2c_smcp.server.base import BaseNamespace


class DummyAuth(AuthenticationProvider):
    async def authenticate(self, sio, environ, auth, headers):
        return True


@pytest.fixture
def ns():
    return BaseNamespace("/ns", DummyAuth())


@pytest.mark.asyncio
async def test_on_connect_success_and_extract_headers_paths(ns):
    server = AsyncMock()
    server.app = MagicMock()

    # 路径1：从 asgi.scope.headers
    environ1 = {"asgi": {"scope": {"headers": [(b"k", b"v")]}}}
    ns.server = server
    ok = await ns.on_connect("sid1", environ1, None)
    assert ok is True

    # 路径2：从 HTTP_HEADERS
    class Auth2(DummyAuth):
        async def authenticate(self, sio, environ, auth, headers):  # noqa: D401
            assert headers == [(b"h1", b"v1")]
            return True

    ns2 = BaseNamespace("/ns", Auth2())
    ns2.server = server
    environ2 = {"HTTP_HEADERS": [(b"h1", b"v1")]}
    ok2 = await ns2.on_connect("sid2", environ2, None)
    assert ok2 is True


@pytest.mark.asyncio
async def test_on_connect_auth_failed(ns):
    class BadAuth(DummyAuth):
        async def authenticate(self, *a, **k):  # noqa: ANN001, D401
            return False

    ns_bad = BaseNamespace("/ns", BadAuth())
    ns_bad.server = AsyncMock()
    with pytest.raises(ConnectionRefusedError):
        await ns_bad.on_connect("sidX", {"asgi": {"scope": {"headers": []}}}, None)


@pytest.mark.asyncio
async def test_on_disconnect_and_trigger_event_translate(ns, monkeypatch):
    ns.server = AsyncMock()
    # rooms: 包含自身房间与其它房间
    ns.rooms = MagicMock(return_value=["sid3", "roomA", "roomB"])  # 自身房间名等于 sid
    ns.leave_room = AsyncMock()
    await ns.on_disconnect("sid3")
    ns.leave_room.assert_any_await("sid3", "roomA")
    ns.leave_room.assert_any_await("sid3", "roomB")

    # trigger_event 将 : 替换为 _，并调用父类 AsyncNamespace.trigger_event
    from socketio import AsyncNamespace

    parent = AsyncMock()
    # 使用 monkeypatch 临时替换，避免污染全局，影响后续集成测试的事件分发
    monkeypatch.setattr(AsyncNamespace, "trigger_event", parent, raising=True)
    await ns.trigger_event("server:join_office", 1, 2)
    parent.assert_awaited_with("server_join_office", 1, 2)



# -*- coding: utf-8 -*-
# filename: test_namespace_sync.py
# @Time    : 2025/09/30 23:42
# @Author  : A2C-SMCP
"""
中文：针对 `a2c_smcp/server/sync_namespace.py` 的同步命名空间集成测试。
English: Integration tests for SyncSMCPNamespace in `a2c_smcp/server/sync_namespace.py`.

说明：
- 仅在本测试包使用的 `_local_sync_server.py` 启动同步 Socket.IO 服务器。
- 使用 werkzeug 在独立进程中运行 WSGI 服务器，彻底解决 GIL 阻塞问题。
"""

import multiprocessing
import socket
import threading
import time
from collections.abc import Generator
from multiprocessing import synchronize
from typing import Any

import pytest
from socketio import Client, Namespace, SimpleClient
from werkzeug.serving import make_server

from a2c_smcp.smcp import (
    ENTER_OFFICE_NOTIFICATION,
    GET_TOOLS_EVENT,
    JOIN_OFFICE_EVENT,
    LEAVE_OFFICE_EVENT,
    LEAVE_OFFICE_NOTIFICATION,
    SMCP_NAMESPACE,
    TOOL_CALL_EVENT,
    UPDATE_CONFIG_EVENT,
)
from tests.integration_tests.server._local_sync_server import create_local_sync_server


@pytest.fixture
def sync_server_port() -> int:
    """
    中文：查找可用端口。
    English: Find an available TCP port.
    """
    with socket.socket() as s:
        s.bind(("127.0.0.1", 0))
        return s.getsockname()[1]


def _run_server_process(port: int, ready_event: synchronize.Event) -> None:
    """在独立进程中运行服务器"""
    try:
        sio, ns, wsgi_app = create_local_sync_server()
        # 禁用监控任务避免关闭时出错
        sio.eio.start_service_task = False

        server = make_server("localhost", port, wsgi_app, threaded=True)

        # 通知主进程服务器已准备好
        ready_event.set()

        # 运行服务器
        server.serve_forever()
    except Exception as e:
        print(f"服务器进程错误: {e}")
        ready_event.set()  # 即使出错也要设置事件，避免主进程无限等待


@pytest.fixture
def startup_and_shutdown_local_sync_server(sync_server_port: int) -> Generator[None, Any, None]:
    # 创建进程间通信事件
    ready_event = multiprocessing.Event()

    # 启动服务器进程
    server_process = multiprocessing.Process(
        target=_run_server_process,
        args=(sync_server_port, ready_event),
        daemon=True,
    )
    server_process.start()

    # 等待服务器准备好
    if not ready_event.wait(timeout=5):
        server_process.terminate()
        server_process.join(timeout=2)
        pytest.fail("服务器进程启动超时")

    try:
        yield
    finally:
        # 终止服务器进程
        if server_process.is_alive():
            server_process.terminate()
            server_process.join(timeout=3)

        # 如果进程仍然存活，强制杀死
        if server_process.is_alive():
            server_process.kill()
            server_process.join(timeout=1)


def _join_office(client: Client | SimpleClient, role: str, office_id: str, name: str) -> None:
    ok, err = (
        client.call(
            JOIN_OFFICE_EVENT,
            {"role": role, "office_id": office_id, "name": name},
            namespace=SMCP_NAMESPACE,
        )
        if isinstance(client, Client)
        else client.call(JOIN_OFFICE_EVENT, {"role": role, "office_id": office_id, "name": name})
    )
    if not (ok and err is None):
        print(f"加入房间失败: role={role}, office_id={office_id}, name={name}, ok={ok}, err={err}")
    assert ok and err is None


def test_enter_and_broadcast_sync(startup_and_shutdown_local_sync_server, sync_server_port: int) -> None:
    agent = Client()
    computer = Client()

    enter_events: list[dict] = []

    @agent.on(ENTER_OFFICE_NOTIFICATION, namespace=SMCP_NAMESPACE)
    def _on_enter(data: dict):  # noqa: ANN001
        enter_events.append(data)

    agent.connect(f"http://localhost:{sync_server_port}", namespaces=[SMCP_NAMESPACE], socketio_path="/socket.io")
    office_id = "office-sync-s1"
    _join_office(agent, role="agent", office_id=office_id, name="robot-S1")

    computer.connect(f"http://localhost:{sync_server_port}", namespaces=[SMCP_NAMESPACE], socketio_path="/socket.io")
    _join_office(computer, role="computer", office_id=office_id, name="comp-S1")

    time.sleep(0.2)
    assert enter_events, "Agent 应收到 ENTER_OFFICE_NOTIFICATION"

    agent.disconnect()
    computer.disconnect()


def test_leave_and_broadcast_sync(startup_and_shutdown_local_sync_server, sync_server_port: int) -> None:
    agent = Client()
    computer = Client()

    leave_events: list[dict] = []

    @agent.on(LEAVE_OFFICE_NOTIFICATION, namespace=SMCP_NAMESPACE)
    def _on_leave(data: dict):  # noqa: ANN001
        leave_events.append(data)

    agent.connect(f"http://localhost:{sync_server_port}", namespaces=[SMCP_NAMESPACE], socketio_path="/socket.io")
    office_id = "office-sync-s2"
    _join_office(agent, role="agent", office_id=office_id, name="robot-S2")

    computer.connect(f"http://localhost:{sync_server_port}", namespaces=[SMCP_NAMESPACE], socketio_path="/socket.io")
    _join_office(computer, role="computer", office_id=office_id, name="comp-S2")

    ok, err = computer.call(LEAVE_OFFICE_EVENT, {"office_id": office_id}, namespace=SMCP_NAMESPACE)
    assert ok and err is None

    time.sleep(0.2)
    assert leave_events, "Agent 应收到 LEAVE_OFFICE_NOTIFICATION"

    agent.disconnect()
    computer.disconnect()


def _run_computer_client_process(port: int, computer_name_queue: multiprocessing.Queue, error_queue: multiprocessing.Queue) -> None:
    """在独立进程中运行Computer客户端"""
    computer = Client()

    @computer.on(GET_TOOLS_EVENT, namespace=SMCP_NAMESPACE)
    def _on_get_tools(data: dict):  # noqa: ANN001
        return {
            "tools": [
                {
                    "name": "echo",
                    "description": "echo text",
                    "params_schema": {"type": "object"},
                    "return_schema": None,
                },
            ],
            "req_id": data["req_id"],
        }

    try:
        computer.connect(f"http://localhost:{port}", namespaces=[SMCP_NAMESPACE], socketio_path="/socket.io")
        office_id = "office-sync-s3"
        computer_name = "comp-S3"
        _join_office(computer, role="computer", office_id=office_id, name=computer_name)

        # 将computer_sid发送给主进程
        computer_name_queue.put(computer_name)

        # 等待并处理GET_TOOLS_EVENT
        computer.wait()
    except Exception as e:
        error_queue.put(f"Computer客户端错误: {str(e)}")
    finally:
        computer.disconnect()


def _run_agent_client_process(
    port: int,
    computer_name: str,
    result_queue: multiprocessing.Queue,
    error_queue: multiprocessing.Queue,
) -> None:
    """在独立进程中运行Agent客户端"""
    try:
        agent = Client()
        agent_id = "robot-S3"
        agent.connect(f"http://localhost:{port}", namespaces=[SMCP_NAMESPACE], socketio_path="/socket.io")
        office_id = "office-sync-s3"
        _join_office(agent, role="agent", office_id=office_id, name=agent_id)

        # 确保连接稳定后再进行调用
        time.sleep(0.2)

        # 执行GET_TOOLS调用
        res = agent.call(
            GET_TOOLS_EVENT,
            {"computer": computer_name, "robot_id": agent_id, "req_id": "req-sync-1"},
            namespace=SMCP_NAMESPACE,
            timeout=15,
        )

        # 将结果发送给主进程
        result_queue.put(res)

        agent.disconnect()
    except Exception as e:
        error_queue.put(f"Agent客户端错误: {str(e)}")


# @pytest.mark.skip
def test_get_tools_success_sync(startup_and_shutdown_local_sync_server: Namespace, sync_server_port: int) -> None:
    """测试同步环境下获取工具列表，使用多进程避免GIL阻塞"""

    # 创建进程间通信队列
    computer_name_queue = multiprocessing.Queue()
    result_queue = multiprocessing.Queue()
    error_queue = multiprocessing.Queue()

    # 1. 启动Computer客户端进程并获取computer_sid
    computer_process = multiprocessing.Process(
        target=_run_computer_client_process,
        args=(sync_server_port, computer_name_queue, error_queue),
        daemon=True,
    )
    computer_process.start()

    try:
        # 等待获取computer_sid
        try:
            computer_name = computer_name_queue.get(timeout=5)
        except Exception:
            # 检查是否有错误
            if not error_queue.empty():
                error_msg = error_queue.get()
                pytest.fail(f"Computer客户端启动失败: {error_msg}")
            else:
                pytest.fail("获取Computer SID超时")

        print(f"获取到Computer NAME: {computer_name}")

        # 2. 启动Agent客户端进程执行工具列表获取
        agent_process = multiprocessing.Process(
            target=_run_agent_client_process,
            args=(sync_server_port, computer_name, result_queue, error_queue),
            daemon=True,
        )
        agent_process.start()

        try:
            # 等待Agent执行结果
            try:
                result = result_queue.get(timeout=20)
                # 验证结果
                assert isinstance(result, dict), f"期望返回dict，实际返回: {type(result)}"
                assert result.get("tools") and result["tools"][0]["name"] == "echo"
            except Exception:
                # 检查是否有错误
                if not error_queue.empty():
                    error_msg = error_queue.get()
                    pytest.fail(f"Agent客户端执行失败: {error_msg}")
                else:
                    pytest.fail("Agent执行超时")
        finally:
            # 清理Agent进程
            if agent_process.is_alive():
                agent_process.terminate()
                agent_process.join(timeout=2)
    finally:
        # 清理Computer进程
        if computer_process.is_alive():
            computer_process.terminate()
            computer_process.join(timeout=2)


def test_update_config_broadcast_sync(startup_and_shutdown_local_sync_server, sync_server_port: int) -> None:
    agent = Client()
    computer = Client()

    received = {"count": 0}

    @agent.on("notify:update_config", namespace=SMCP_NAMESPACE)
    def _on_update(data: dict):  # noqa: ANN001
        received["count"] += 1

    agent.connect(f"http://localhost:{sync_server_port}", namespaces=[SMCP_NAMESPACE], socketio_path="/socket.io")
    office_id = "office-sync-s4"
    _join_office(agent, role="agent", office_id=office_id, name="robot-S4")

    computer.connect(f"http://localhost:{sync_server_port}", namespaces=[SMCP_NAMESPACE], socketio_path="/socket.io")
    _join_office(computer, role="computer", office_id=office_id, name="comp-S4")

    computer.call(UPDATE_CONFIG_EVENT, {"computer": computer.get_sid(namespace=SMCP_NAMESPACE)}, namespace=SMCP_NAMESPACE)

    time.sleep(0.2)
    assert received["count"] >= 1

    agent.disconnect()
    computer.disconnect()


def test_tool_call_forward_sync(startup_and_shutdown_local_sync_server, sync_server_port: int) -> None:
    """测试同步环境下工具调用转发，使用多线程避免阻塞"""
    agent = Client()
    computer = Client()

    received = {"count": 0, "data": None}
    call_result: dict = {"error": None}
    # 用于同步的事件
    computer_ready = threading.Event()
    call_completed = threading.Event()

    @computer.on(TOOL_CALL_EVENT, namespace=SMCP_NAMESPACE)
    def _on_tool_call(data: dict):  # noqa: ANN001
        received["count"] += 1
        received["data"] = data
        # 返回响应给 Agent
        return {"ok": True, "echo": data}

    def run_computer_client():
        """在独立线程中运行Computer客户端"""
        try:
            computer.connect(f"http://localhost:{sync_server_port}", namespaces=[SMCP_NAMESPACE], socketio_path="/socket.io")
            office_id = "office-sync-s5"
            _join_office(computer, role="computer", office_id=office_id, name="comp-S5")
            computer_ready.set()  # 通知Computer客户端已准备好

            # 等待调用完成
            call_completed.wait(timeout=20)
        except Exception as e:
            call_result["error"] = f"Computer客户端错误: {str(e)}"
            computer_ready.set()
        finally:
            try:
                computer.disconnect()
            except Exception:
                pass

    # 先连接Agent客户端
    agent.connect(f"http://localhost:{sync_server_port}", namespaces=[SMCP_NAMESPACE], socketio_path="/socket.io")
    office_id = "office-sync-s5"
    _join_office(agent, role="agent", office_id=office_id, name="robot-S5")

    # 启动Computer客户端线程
    computer_thread = threading.Thread(target=run_computer_client, daemon=True)
    computer_thread.start()

    try:
        # 等待Computer客户端准备好
        if not computer_ready.wait(timeout=10):
            pytest.fail("Computer客户端连接超时")

        if call_result["error"]:
            pytest.fail(call_result["error"])

        # 确保Computer客户端完全连接后再进行调用
        time.sleep(0.2)

        # 执行Agent工具调用
        res = agent.call(
            TOOL_CALL_EVENT,
            {
                "robot_id": "robot-S5",
                "computer": "comp-S5",
                "tool_name": "echo",
                "params": {"text": "hi"},
                "req_id": "req-sync-2",
                "timeout": 5,
            },
            namespace=SMCP_NAMESPACE,
            timeout=15,
        )

        # 同步命名空间现在使用 call 方法，等待 Computer 响应
        assert isinstance(res, dict), f"期望返回 dict，实际返回: {type(res)}"
        assert res.get("ok") is True, f"期望 ok=True，实际返回: {res}"
        assert res.get("echo") is not None, f"期望有 echo 字段，实际返回: {res}"

        # 验证 Computer 收到了工具调用
        assert received["count"] == 1, f"Computer应该收到1次工具调用事件，实际收到{received['count']}次"
        assert received["data"] is not None
        assert received["data"]["tool_name"] == "echo"
        assert received["data"]["params"]["text"] == "hi"

    finally:
        call_completed.set()
        computer_thread.join(timeout=5)
        agent.disconnect()



"""
* 文件名: sync_namespace
* 作者: JQQ
* 创建日期: 2025/9/29
* 最后修改日期: 2025/9/29
* 版权: 2023 JQQ. All rights reserved.
* 依赖: socketio, loguru, pydantic
* 描述: 同步版本SMCP协议Namespace实现 / Synchronous SMCP protocol Namespace implementation
"""

import copy
from typing import cast

from pydantic import TypeAdapter

from a2c_smcp.server.sync_auth import SyncAuthenticationProvider
from a2c_smcp.server.sync_base import SyncBaseNamespace
from a2c_smcp.server.types import OFFICE_ID, SID
from a2c_smcp.server.utils import get_all_sessions_in_office
from a2c_smcp.smcp import (
    CANCEL_TOOL_CALL_NOTIFICATION,
    ENTER_OFFICE_NOTIFICATION,
    GET_DESKTOP_EVENT,
    GET_TOOLS_EVENT,
    LEAVE_OFFICE_NOTIFICATION,
    SMCP_NAMESPACE,
    TOOL_CALL_EVENT,
    UPDATE_CONFIG_NOTIFICATION,
    UPDATE_DESKTOP_NOTIFICATION,
    UPDATE_TOOL_LIST_NOTIFICATION,
    AgentCallData,
    EnterOfficeNotification,
    EnterOfficeReq,
    GetDeskTopReq,
    GetDeskTopRet,
    GetToolsReq,
    GetToolsRet,
    LeaveOfficeNotification,
    LeaveOfficeReq,
    ListRoomReq,
    ListRoomRet,
    SessionInfo,
    UpdateComputerConfigReq,
    UpdateMCPConfigNotification,
)
from a2c_smcp.utils.logger import logger


class SyncSMCPNamespace(SyncBaseNamespace):
    """
    同步SMCP命名空间，处理SMCP相关事件（同步）
    Synchronous Socket.IO namespace for handling SMCP-related events
    """

    def __init__(self, auth_provider: SyncAuthenticationProvider) -> None:
        """
        初始化SMCP命名空间（同步）
        Initialize SMCP namespace (sync)
        """
        super().__init__(namespace=SMCP_NAMESPACE, auth_provider=auth_provider)

    def enter_room(self, sid: SID, room: OFFICE_ID, namespace: str | None = None) -> None:
        """
        客户端加入房间，维护session中的sid/name/office_id字段（同步）
        Client joins room, maintain sid/name/office_id in session (sync)
        """
        session = self.get_session(sid)

        if session.get("sid") != sid:
            session["sid"] = sid
        if not session.get("name"):
            session["name"] = f"{session.get('role', 'unknown')}_{sid[:6]}"

        if session.get("role") == "agent":
            if session.get("office_id") and session.get("office_id") != room:
                logger.error(f"Agent sid: {sid} already in room: {session.get('office_id')}, can't join room: {room}")
                raise ValueError("Agent sid already in room")
            elif not session.get("office_id"):
                for participant_sid, _participant_eio_sid in self.server.manager.get_participants(SMCP_NAMESPACE, room):
                    participant_session = self.get_session(participant_sid)
                    if participant_session.get("role") == "agent":
                        raise ValueError("Agent already in room")
            else:
                logger.warning(
                    f"Agent sid: {sid} already in room: {session.get('office_id')}. 正在重复加入房间",
                )
                return
        else:
            if session.get("office_id") and (past_room := session.get("office_id")) != room:
                self.leave_room(sid, past_room)
            elif session.get("office_id") == room:
                logger.warning(
                    f"Computer sid: {sid} already in room: {session.get('office_id')}. 正在重复加入房间",
                )
                return

        super().enter_room(sid, room)
        session["office_id"] = room
        self.save_session(sid, session)

        # 注册name到sid的映射
        # Register name-to-sid mapping
        self._register_name(session["name"], sid)

        # 根据角色发送不同的通知 / Send different notifications based on role
        notification_data: EnterOfficeNotification = {"office_id": room}
        if session.get("role") == "computer":
            notification_data["computer"] = session.get("name")
        else:
            notification_data["agent"] = session.get("name")

        self.emit(
            ENTER_OFFICE_NOTIFICATION,
            notification_data,
            skip_sid=sid,
            room=room,
        )

    def leave_room(self, sid: SID, room: OFFICE_ID, namespace: str | None = None) -> None:
        """
        在离开房间之前发布离开消息（同步）
        Publish leave message before leaving room (sync)
        """
        session = self.get_session(sid)

        # 构建离开通知，使用name而不是sid
        # Build leave notification using name instead of sid
        client_name = session.get("name", "")
        notification = (
            LeaveOfficeNotification(office_id=room, computer=client_name)
            if session.get("role") == "computer"
            else LeaveOfficeNotification(office_id=room, agent=client_name)
        )
        self.emit(LEAVE_OFFICE_NOTIFICATION, notification, skip_sid=sid, room=room)

        # 注销name映射
        # Unregister name mapping
        self._unregister_name(sid)

        if "office_id" in session:
            del session["office_id"]
        self.save_session(sid, session)

        super().leave_room(sid, room)

    def on_server_join_office(self, sid: str, data: EnterOfficeReq) -> tuple[bool, str | None]:
        """
        同步：Computer/Agent加入房间
        Sync: Computer or Agent joins room
        """
        role_info = TypeAdapter(EnterOfficeReq).validate_python(data)
        expected_role = role_info["role"]

        session = self.get_session(sid)
        backup_session = copy.deepcopy(session)

        try:
            if session.get("role") and session["role"] != expected_role:
                return False, f"Role mismatch, expected {expected_role}, but {session['role']} use this sid exists"

            session["role"] = expected_role
            session["name"] = role_info["name"]
            self.save_session(sid, session)

            self.enter_room(sid, role_info["office_id"])
            return True, None
        except Exception as e:
            self.save_session(sid, backup_session)
            return False, f"Internal server error: {str(e)}"

    def on_server_leave_office(self, sid: str, data: LeaveOfficeReq) -> tuple[bool, str | None]:
        """
        同步：Computer/Agent离开房间
        Sync: Computer or Agent leaves room
        """
        try:
            self.leave_room(sid, data["office_id"])
            return True, None
        except Exception as e:
            return False, f"Internal server error: {str(e)}"

    def on_server_tool_call_cancel(self, sid: str, data: AgentCallData) -> None:
        """
        同步：广播取消ToolCall到房间内的其他成员
        Sync: broadcast tool call cancellation to other members in the room
        """
        session = self.get_session(sid)
        assert session["role"] == "agent", "目前仅支持Agent调用取消ToolCall的操作"

        agent_call = TypeAdapter(AgentCallData).validate_python(data)
        assert session.get("name") == agent_call["agent"], "取消工具调用的广播仅可以由对应Agent发出"

        # 广播到 office 房间，而不是 Agent 的私有房间 / Broadcast to office room, not Agent's private room
        office_id = session.get("office_id")
        self.emit(
            CANCEL_TOOL_CALL_NOTIFICATION,
            agent_call,
            room=office_id,
            skip_sid=sid,
        )

    def on_server_update_config(self, sid: str, data: UpdateComputerConfigReq) -> None:
        """
        同步：广播更新MCP配置
        Sync: broadcast MCP config update
        """
        session = self.get_session(sid)
        assert session["role"] == "computer", "目前仅支持Computer调用更新MCP配置的操作"

        update_config = TypeAdapter(UpdateComputerConfigReq).validate_python(data)
        self.emit(
            UPDATE_CONFIG_NOTIFICATION,
            UpdateMCPConfigNotification(computer=update_config["computer"]),
            room=session["office_id"],
            skip_sid=sid,
        )

    def on_server_update_tool_list(self, sid: str, data: UpdateComputerConfigReq) -> None:
        """
        同步：广播工具列表更新
        Sync: broadcast tool list update
        """
        session = self.get_session(sid)
        assert session["role"] == "computer", "目前仅支持Computer上报工具列表变更"

        update_req = TypeAdapter(UpdateComputerConfigReq).validate_python(data)

        self.emit(
            UPDATE_TOOL_LIST_NOTIFICATION,
            {"computer": update_req["computer"]},
            room=session.get("office_id"),
            skip_sid=sid,
        )

    def on_client_tool_call(self, sid: str, data: dict) -> dict:
        """
        同步：响应工具调用，使用 call 方法等待 Computer 返回结果
        Sync: respond to tool call, use call method to wait for Computer response
        """
        session = self.get_session(sid)
        assert session["role"] == "agent", "目前仅支持Agent调用工具"

        tool_call = TypeAdapter(dict).validate_python(data)

        # 通过name获取computer的sid
        # Get computer's sid by name
        computer_name = tool_call["computer"]
        computer_sid = self.get_sid_by_name(computer_name)
        if not computer_sid:
            raise ValueError(f"Computer with name '{computer_name}' not found")

        # 使用 call 方法调用 Computer，等待返回结果 / Use call method to invoke Computer and wait for result
        return cast(
            dict,
            self.call(
                TOOL_CALL_EVENT,
                tool_call,
                to=computer_sid,
                namespace=SMCP_NAMESPACE,
            ),
        )

    def on_client_get_tools(self, sid: str, data: GetToolsReq) -> GetToolsRet:
        """
        同步：获取指定Computer的工具列表（使用 Socket.IO 的 call 等待客户端返回）
        Sync: get tool list of specified Computer using Socket.IO call
        """
        computer_name = data["computer"]

        # 通过name获取computer的sid
        # Get computer's sid by name
        computer_sid = self.get_sid_by_name(computer_name)
        if not computer_sid:
            raise ValueError(f"Computer with name '{computer_name}' not found")

        session = self.get_session(computer_sid)
        assert session["role"] == "computer", "目前仅支持Computer获取工具列表"

        agent_session = self.get_session(sid)
        computer_office_id = session.get("office_id")
        agent_office_id = agent_session.get("office_id")
        assert computer_office_id == agent_office_id, "目前仅支持Agent获取自己房间内Computer的工具列表"

        client_response = self.call(
            GET_TOOLS_EVENT,
            data,
            to=computer_sid,
            namespace=SMCP_NAMESPACE,
        )

        return TypeAdapter(GetToolsRet).validate_python(client_response)

    def on_client_get_desktop(self, sid: str, data: GetDeskTopReq) -> GetDeskTopRet:
        """
        同步：获取指定Computer的桌面信息（窗口组织后的视图）
        Sync: get desktop view from specified Computer

        要求：Agent 与 Computer 需在同一 office / Requirement: Agent and Computer must be in the same office
        """
        computer_name = data["computer"]

        # 通过name获取computer的sid
        # Get computer's sid by name
        computer_sid = self.get_sid_by_name(computer_name)
        if not computer_sid:
            raise ValueError(f"Computer with name '{computer_name}' not found")

        session = self.get_session(computer_sid)
        assert session["role"] == "computer", "目前仅支持Computer获取桌面"

        agent_session = self.get_session(sid)
        computer_office_id = session.get("office_id")
        agent_office_id = agent_session.get("office_id")
        assert computer_office_id == agent_office_id, "目前仅支持Agent获取自己房间内Computer的桌面"

        client_response = self.call(
            GET_DESKTOP_EVENT,
            data,
            to=computer_sid,
            namespace=SMCP_NAMESPACE,
        )

        return TypeAdapter(GetDeskTopRet).validate_python(client_response)

    def on_server_update_desktop(self, sid: str, data: UpdateComputerConfigReq) -> None:
        """
        同步：将事件广播至对应的房间内其他参与者，通知桌面刷新
        Sync: broadcast to others in the room to notify desktop update

        Args:
            sid (str): 发起者ID，应为Computer / Initiator ID, should be Computer
            data (UpdateComputerConfigReq): 载荷复用 UpdateConfigReq，仅需 computer 标识
        """
        session = self.get_session(sid)
        assert session["role"] == "computer", "目前仅支持Computer上报桌面刷新"

        update_req = TypeAdapter(UpdateComputerConfigReq).validate_python(data)
        self.emit(
            UPDATE_DESKTOP_NOTIFICATION,
            {"computer": update_req["computer"]},
            room=session.get("office_id"),
            skip_sid=sid,
        )

    def on_server_list_room(self, sid: str, data: ListRoomReq) -> ListRoomRet:
        """
        同步：列出指定房间内的所有会话信息。Agent可以通过此事件查询房间内的所有Computer和Agent。
        Sync: List all sessions in the specified room. Agent can query all Computers and Agents in the room via this event.

        Args:
            sid (str): 发起者ID，一般是Agent / Initiator ID, usually Agent
            data (ListRoomReq): 列出房间请求数据，包含office_id和req_id / List room request data with office_id and req_id

        Returns:
            ListRoomRet: 房间内所有会话信息列表 / List of all session info in the room
        """
        # 验证请求数据 / Validate request data
        list_room_req = TypeAdapter(ListRoomReq).validate_python(data)
        office_id = list_room_req["office_id"]
        req_id = list_room_req["req_id"]

        # 验证发起者权限：确保Agent在请求的房间内 / Verify initiator permission: ensure Agent is in the requested room
        agent_session = self.get_session(sid)
        agent_office_id = agent_session.get("office_id")

        assert agent_office_id == office_id, f"Agent只能查询自己所在房间的会话信息。Agent office: {agent_office_id}, requested: {office_id}"

        # 使用工具函数获取房间内所有会话信息 / Use utility function to get all session info in the room
        all_sessions = get_all_sessions_in_office(office_id, self.server)

        # 转换为SessionInfo格式 / Convert to SessionInfo format
        sessions: list[SessionInfo] = []
        for session in all_sessions:
            if session.get("role") in ["computer", "agent"]:
                session_info: SessionInfo = {
                    "sid": session.get("sid", ""),
                    "name": session.get("name", ""),
                    "role": session["role"],
                    "office_id": session.get("office_id", ""),
                }
                sessions.append(session_info)

        return ListRoomRet(sessions=sessions, req_id=req_id)



# -*- coding: utf-8 -*-
# 文件名: base.py
# 作者: JQQ
# 创建日期: 2025/9/24
# 最后修改日期: 2025/9/24
# 版权: 2023 JQQ. All rights reserved.
# 依赖: typing, a2c_smcp_cc.mcp_clients.model
# 描述:
#   中文: 定义计算机管理器的泛型基类，抽象出会话（Session）类型以便于不同交互环境（如 CLI、GUI、Web）复用核心能力。
#   English: Define a generic base class for computer managers, abstracting the Session type so that core
#            capabilities can be reused across different interaction environments (CLI, GUI, Web).

"""
中文: 计算机管理器泛型基类
English: Generic base class for computer manager.

说明 Notes:
- 该基类只规定与 Session 相关的关键方法签名，将 Session 抽象为泛型参数 S。
- 具体实现（如 CLI 版）在子类中特化 S 的具体类型，并实现这些方法。
- 其他与 Session 无关的方法（状态查询、工具执行等）可由子类自由扩展。
"""

from __future__ import annotations

from abc import ABC, abstractmethod
from typing import Any, Generic, TypeVar

from a2c_smcp.computer.mcp_clients.model import MCPServerConfig, MCPServerInput

S = TypeVar("S")


class BaseComputer(Generic[S], ABC):
    """
    中文: 计算机管理器的抽象基类，通过泛型 S 抽象会话类型。
    English: Abstract base class for computer manager, parameterized by generic session type S.
    """

    @abstractmethod
    async def boot_up(self, *, session: S | None = None) -> None:  # pragma: no cover - interface definition
        """
        中文: 启动并初始化资源；可选地使用给定的会话。
        English: Boot and initialize resources; optionally use the given session.
        """
        raise NotImplementedError

    @abstractmethod
    async def aadd_or_aupdate_server(self, server: MCPServerConfig | dict, *, session: S | None = None) -> None:  # pragma: no cover
        """
        中文: 动态添加或更新某个服务配置。
        English: Add or update a server config dynamically.
        """
        raise NotImplementedError

    # -------- MCP Server lifecycle --------
    @abstractmethod
    async def aremove_server(self, server_name: str, *, session: S | None = None) -> None:  # pragma: no cover
        """
        中文: 动态移除某个服务配置。
        English: Remove a server config dynamically.
        """
        raise NotImplementedError

    # -------- Inputs definition management --------
    @abstractmethod
    def update_inputs(self, inputs: set[MCPServerInput], *, session: S | None = None) -> None:  # pragma: no cover
        """
        中文: 批量更新 inputs 定义，并重置解析器（可带入默认会话）。
        English: Update inputs definitions and reset resolver (with optional default session).
        """
        raise NotImplementedError

    @abstractmethod
    def add_or_update_input(self, input_cfg: MCPServerInput, *, session: S | None = None) -> None:  # pragma: no cover
        """
        中文: 新增或更新单个 input 定义。
        English: Add or update a single input definition.
        """
        raise NotImplementedError

    @abstractmethod
    def remove_input(self, input_id: str, *, session: S | None = None) -> bool:  # pragma: no cover
        """
        中文: 按 id 移除单个 input 定义，返回是否删除发生。
        English: Remove a single input definition by id, return whether deletion happened.
        """
        raise NotImplementedError

    @abstractmethod
    def get_input(self, input_id: str, *, session: S | None = None) -> MCPServerInput | None:  # pragma: no cover
        """
        中文: 获取指定 id 的 input 定义。
        English: Get input definition by id.
        """
        raise NotImplementedError

    @abstractmethod
    def list_inputs(self, *, session: S | None = None) -> tuple[MCPServerInput, ...]:  # pragma: no cover
        """
        中文: 列出当前全部 inputs（不可变视图）。
        English: List all current inputs (immutable view).
        """
        raise NotImplementedError

    # -------- Current input values (cache) --------
    @abstractmethod
    def get_input_value(self, input_id: str, *, session: S | None = None) -> Any | None:  # pragma: no cover
        """
        中文: 获取指定 id 的当前值（缓存）。
        English: Get current value for the given id (cache).
        """
        raise NotImplementedError

    @abstractmethod
    def set_input_value(self, input_id: str, value: Any, *, session: S | None = None) -> bool:  # pragma: no cover
        """
        中文: 设置指定 id 的当前值（缓存）。
        English: Set current value for the given id (cache).
        """
        raise NotImplementedError

    @abstractmethod
    def remove_input_value(self, input_id: str, *, session: S | None = None) -> bool:  # pragma: no cover
        """
        中文: 删除指定 id 的当前值（缓存）。
        English: Delete current value for the given id (cache).
        """
        raise NotImplementedError

    @abstractmethod
    def list_input_values(self, *, session: S | None = None) -> dict[str, Any]:  # pragma: no cover
        """
        中文: 列出所有已解析的 inputs 当前值（缓存快照）。
        English: List all resolved input values (cache snapshot).
        """
        raise NotImplementedError

    @abstractmethod
    def clear_input_values(self, input_id: str | None = None, *, session: S | None = None) -> None:  # pragma: no cover
        """
        中文: 清空所有或指定 id 的输入值缓存。
        English: Clear all cached values or the specified id.
        """
        raise NotImplementedError

    # -------- Lifecycle --------
    @abstractmethod
    async def shutdown(self, *, session: S | None = None) -> None:  # pragma: no cover
        """
        中文: 关闭资源。
        English: Shutdown resources.
        """
        raise NotImplementedError



# -*- coding: utf-8 -*-
# filename: direct_execution.py
# @Time    : 2025/8/19 11:11
# @Author  : JQQ
# @Email   : jqq1716@gmail.com
# @Software: PyCharm
"""Copy from https://github.com/modelcontextprotocol/python-sdk/blob/main/examples/snippets/servers/direct_execution.py
Example showing direct execution of an MCP server.

This is the simplest way to run an MCP server directly.
cd to the `examples/snippets` directory and run:
    python servers/direct_execution.py

注意此进程启动后，如果要关闭，需要使用 Ctrl+D 而不是 Ctrl+C。本质上是通过 Ctrl+D 退出Stdin。从而达到关闭进程的目的
这个效果是MCP Server封装的默认行为。
"""

from mcp.server.fastmcp import FastMCP

mcp = FastMCP("My App")


@mcp.tool()
def hello(name: str = "World") -> str:
    """Say hello to someone."""
    return f"Hello, {name}!"


def main():
    """Entry point for the direct execution server."""
    print("starting...")
    mcp.run()


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-
# filename: resources_subscribe_stdio_server.py
# @Time    : 2025/10/02 17:01
# @Author  : JQQ
# @Email   : jqq1716@gmail.com
# @Software: PyCharm
"""
中文: 提供 Resources 能力并支持 Subscribe 的 MCP Stdio 服务器，用于 Desktop 集成测试。
英文: MCP Stdio server exposing Resources with Subscribe support for Desktop integration tests.

文件名: resources_subscribe_stdio_server.py / filename: resources_subscribe_stdio_server.py
作者: JQQ / author: JQQ
创建日期: 2025/10/02 / created at: 2025/10/02
最后修改日期: 2025/10/02 / last modified at: 2025/10/02
版权: 2023 JQQ. All rights reserved. / copyright: 2023 JQQ. All rights reserved.
依赖: anyio, mcp / dependencies: anyio, mcp
描述: 暴露 window:// 资源，且在 capabilities 中声明 subscribe=True；订阅后立即发送一次更新通知。
      Expose window:// resources and declare subscribe=True; send one ResourceUpdated notification on subscribe.
"""

from __future__ import annotations

import anyio
from mcp import types
from mcp.server.lowlevel.server import Server as LowLevelServer
from mcp.server.stdio import stdio_server


class TestServer(LowLevelServer):
    """
    中文: 测试子类，覆盖 capabilities，使得当注册了订阅处理器时 resources.subscribe=True。
    英文: Test subclass overriding capabilities to set resources.subscribe=True when subscribe handlers exist.
    """

    def get_capabilities(  # type: ignore[override]
        self,
        notification_options,
        experimental_capabilities,
    ) -> types.ServerCapabilities:
        caps = super().get_capabilities(notification_options, experimental_capabilities)
        # 如果注册了订阅处理器，则开启 subscribe 标记 / enable subscribe flag if handlers exist
        if types.SubscribeRequest in self.request_handlers or types.UnsubscribeRequest in self.request_handlers:
            resources_cap = caps.resources or types.ResourcesCapability(
                subscribe=False,
                listChanged=False,
            )
            caps = types.ServerCapabilities(
                prompts=caps.prompts,
                resources=types.ResourcesCapability(
                    subscribe=True,
                    listChanged=resources_cap.listChanged,
                ),
                tools=caps.tools,
                logging=caps.logging,
                experimental=caps.experimental,
                completions=caps.completions,
            )
        return caps


# 预置若干窗口资源 / preset window resources
WINDOW_HOST = "example.desktop.subscribe.a"
WINDOW_RESOURCES: list[types.Resource] = [
    types.Resource(
        uri=f"window://{WINDOW_HOST}/main?priority=60",
        name="Main Window",
        description="中文: 主窗口; 英文: Main window",
        mimeType="text/markdown",
    ),
    types.Resource(
        uri=f"window://{WINDOW_HOST}/dashboard?priority=90&fullscreen=true",
        name="Dashboard",
        description="中文: 仪表盘; 英文: Dashboard",
        mimeType="text/markdown",
    ),
]


async def run() -> None:
    """
    中文: 启动支持 Resources + Subscribe 的服务器，提供 window:// 的列举、读取与订阅更新。
    英文: Start a server with Resources + Subscribe, providing window:// list/read/subscribe updates.
    """
    server = TestServer(name="itest-resources-subscribe", version="0.0.1", instructions="itest-desktop")

    # 列举资源（分页模拟）/ list resources with pagination simulation
    @server.list_resources()
    async def list_resources(req: types.ListResourcesRequest | None = None) -> list[types.Resource]:
        """
        中文: 模拟真实服务的分页返回；第一页返回一条并携带 nextCursor='page2'，第二页返回剩余条目。
        英文: Mimic real server pagination; page1 returns one item with nextCursor='page2', page2 returns the rest.
        """
        return WINDOW_RESOURCES

    # 读取资源内容 / read resource contents
    @server.read_resource()
    async def read_resource(uri: types.AnyUrl):
        text = (
            f"# Window Resource (subscribe)\n\nURI: {uri}\n\n中文: 这是订阅版A测试窗口内容。\n"
            f"英文: This is subscribed-A test window content.\n"
        )
        return text

    # 工具列表与调用处理器 / tools listing and calling handlers
    # 中文: 提供一个简单工具 mark_a，便于在测试中通过 tc 切换“最近调用服务”为 A。
    # 英文: Provide a simple tool `mark_a` so tests can switch the "most-recently-called server" to A via tc.
    @server.list_tools()
    async def list_tools() -> list[types.Tool]:
        return [
            types.Tool(
                name="mark_a",
                description="中文: 标记A; 英文: mark A",
                inputSchema={"type": "object", "properties": {}},
            ),
        ]

    @server.call_tool()
    async def call_tool(name: str, arguments: dict | None):  # noqa: ARG001
        # 中文: 回显工具名，便于 CLI 端确认调用已发生。
        # 英文: Echo tool name so the CLI can confirm the call happened.
        return [types.TextContent(type="text", text=f"ok:{name}")]

    # 订阅与取消订阅 / subscribe and unsubscribe
    @server.subscribe_resource()
    async def subscribe(uri: types.AnyUrl):
        # 订阅后立即发送一次更新通知，便于测试端验证 / send one update right after subscribing
        ctx = server.request_context
        await ctx.session.send_resource_updated(uri)

    @server.unsubscribe_resource()
    async def unsubscribe(_: types.AnyUrl):
        # 此处无需额外处理 / no-op for tests
        return None

    async with stdio_server() as (read_stream, write_stream):
        init_opts = server.create_initialization_options()
        await server.run(read_stream, write_stream, init_opts)


if __name__ == "__main__":
    anyio.run(run)



# -*- coding: utf-8 -*-
# filename: test_organize.py
# @Time    : 2025/10/02 16:27
# @Author  : JQQ
# @Email   : jqq1716@gmail.com
# @Software: PyCharm
"""
组织策略单元测试
Unit tests for desktop organizing policy

仅测试 organize_desktop 的行为，不依赖 Computer。
Only verify organize_desktop behavior, decoupled from Computer.
"""

from types import SimpleNamespace

import pytest
from mcp.types import ReadResourceResult, TextResourceContents

from a2c_smcp.computer.desktop.organize import organize_desktop


@pytest.mark.asyncio
async def test_priority_within_server_and_size_cap():
    """
    - 同一服务器内按 priority 降序
    - 全局 size 截断
    """
    # 资源详情：每个窗口都有一段不同的文本，便于断言渲染
    d1 = ReadResourceResult(contents=[TextResourceContents(text="w1-text", uri="window://srv/w1?priority=10")])
    d2 = ReadResourceResult(contents=[TextResourceContents(text="w2-text", uri="window://srv/w2?priority=90")])
    d3 = ReadResourceResult(contents=[TextResourceContents(text="w3-text", uri="window://srv/w3")])

    windows = [
        ("srv", SimpleNamespace(uri="window://srv/w1?priority=10"), d1),
        ("srv", SimpleNamespace(uri="window://srv/w2?priority=90"), d2),
        ("srv", SimpleNamespace(uri="window://srv/w3"), d3),  # 默认0
    ]
    ret = await organize_desktop(windows=windows, size=2, history=tuple())
    # 应优先 w2，再 w1；且渲染包含文本内容
    assert ret[0].startswith("window://srv/w2?priority=90") and "w2-text" in ret[0]
    assert ret[1].startswith("window://srv/w1?priority=10") and "w1-text" in ret[1]


@pytest.mark.asyncio
async def test_fullscreen_one_per_server_then_next_server():
    """
    - 若遇到 fullscreen=True 的窗口，则该 MCP 仅推入这一个；然后进入下一个 MCP
    """
    d_a1 = ReadResourceResult(contents=[TextResourceContents(uri="window://A/a1?priority=50", text="a1")])
    d_a2 = ReadResourceResult(contents=[TextResourceContents(uri="window://A/a2?fullscreen=true&priority=10", text="a2-full")])
    d_a3 = ReadResourceResult(contents=[TextResourceContents(uri="window://A/a3?priority=90", text="a3")])
    d_b1 = ReadResourceResult(contents=[TextResourceContents(uri="window://B/b1?priority=5", text="b1")])
    windows = [
        ("A", SimpleNamespace(uri="window://A/a1?priority=50"), d_a1),
        ("A", SimpleNamespace(uri="window://A/a2?fullscreen=true&priority=10"), d_a2),
        ("A", SimpleNamespace(uri="window://A/a3?priority=90"), d_a3),
        ("B", SimpleNamespace(uri="window://B/b1?priority=5"), d_b1),
    ]
    # history 让 A 在前
    history = ({"server": "A"},)
    ret = await organize_desktop(windows=windows, size=None, history=history)
    # A 只应输出 fullscreen 的 a2，然后进入 B
    assert ret[0].startswith("window://A/a2?fullscreen=true&priority=10") and "a2-full" in ret[0]
    assert any(x.startswith("window://B/b1?priority=5") and "b1" in x for x in ret)  # B 的内容随后加入


@pytest.mark.asyncio
async def test_server_order_by_recent_history():
    """
    - 服务器顺序按最近历史倒序优先
    """
    d_a = ReadResourceResult(contents=[TextResourceContents(uri="window://A/a1?priority=1", text="a")])
    d_b = ReadResourceResult(contents=[TextResourceContents(uri="window://B/b1?priority=1", text="b")])
    d_c = ReadResourceResult(contents=[TextResourceContents(uri="window://C/c1?priority=1", text="c")])
    windows = [
        ("A", SimpleNamespace(uri="window://A/a1?priority=1"), d_a),
        ("B", SimpleNamespace(uri="window://B/b1?priority=1"), d_b),
        ("C", SimpleNamespace(uri="window://C/c1?priority=1"), d_c),
    ]
    # 最近使用顺序：C -> A（B 未使用）
    history = (
        {"server": "A"},
        {"server": "C"},
    )
    ret = await organize_desktop(windows=windows, size=None, history=history)
    # C 在 A 前，剩余 B 按名称排序追加（B）
    assert ret[0].startswith("window://C/c1?priority=1") and "c" in ret[0]
    assert ret[1].startswith("window://A/a1?priority=1") and "a" in ret[1]
    assert ret[2].startswith("window://B/b1?priority=1") and "b" in ret[2]


@pytest.mark.asyncio
async def test_size_zero_returns_empty():
    ret = await organize_desktop(windows=[], size=0, history=tuple())
    assert ret == []


@pytest.mark.asyncio
async def test_skip_empty_contents_and_detail_exception():
    """
    空内容与 detail.contents 访问异常会被跳过。
    Empty contents and exception when accessing detail.contents should be skipped.
    """
    from types import SimpleNamespace as NS

    from mcp.types import ReadResourceResult, TextResourceContents

    # 空 contents -> 被跳过
    empty_detail = ReadResourceResult(contents=[])
    # 访问 contents 抛出异常 -> 被跳过

    class BadDetail:
        def __getattribute__(self, name):
            if name == "contents":
                raise RuntimeError("boom")
            return super().__getattribute__(name)

    ok_detail = ReadResourceResult(contents=[TextResourceContents(text="ok", uri="window://S/ok")])

    windows = [
        ("S", NS(uri="window://S/empty"), empty_detail),
        ("S", NS(uri="window://S/bad"), BadDetail()),
        ("S", NS(uri="window://S/ok"), ok_detail),
    ]
    ret = await organize_desktop(windows=windows, size=None, history=tuple())
    # 仅保留 ok 窗口
    assert len(ret) == 1 and ret[0].startswith("window://S/ok") and "ok" in ret[0]


@pytest.mark.asyncio
async def test_invalid_window_uri_is_skipped():
    """
    非法的 URI 解析失败会被跳过。
    Invalid WindowURI should be skipped when parsing fails.
    """
    from types import SimpleNamespace as NS

    from mcp.types import ReadResourceResult, TextResourceContents

    bad_detail = ReadResourceResult(contents=[TextResourceContents(text="bad", uri="bad://whatever")])
    good_detail = ReadResourceResult(contents=[TextResourceContents(text="good", uri="window://G/good")])

    windows = [
        ("G", NS(uri=":::this_is_not_a_uri"), bad_detail),  # 将触发 WindowURI 解析失败
        ("G", NS(uri="window://G/good"), good_detail),
    ]
    ret = await organize_desktop(windows=windows, size=None, history=tuple())
    # 仅包含合法 URI
    assert len(ret) == 1 and ret[0].startswith("window://G/good") and "good" in ret[0]


@pytest.mark.asyncio
async def test_render_blob_and_unknown_and_render_exception():
    """
    渲染分支：
    - BlobResourceContents -> 仅记录日志，不加入文本
    - 未知类型 -> 仅记录错误日志
    - 渲染异常 -> 捕获并回退为 URI 字符串
    Rendering branches: Blob, unknown type, and exception fallback.
    """
    from types import SimpleNamespace as NS

    from mcp.types import BlobResourceContents, ReadResourceResult, TextResourceContents

    # Blob + 文本混合 -> 返回包含 URI 与文本（Blob 不追加文本）
    detail_blob = ReadResourceResult(
        contents=[
            BlobResourceContents(uri="window://R/blob", mimeType="application/octet-stream", blob="eHg="),
            TextResourceContents(uri="window://R/blob", text="t1"),
        ],
    )
    # 为了触发 _render 的未知类型分支，先创建合法实例，再在运行期覆盖 contents
    detail_unknown = ReadResourceResult(contents=[TextResourceContents(uri="window://R/u", text="t2")])
    detail_unknown.contents = [object(), TextResourceContents(uri="window://R/u", text="t2")]
    # 渲染异常：contents 不是可迭代对象 -> TypeError -> except 分支返回纯 URI
    detail_exception = ReadResourceResult(contents=[TextResourceContents(uri="window://R/ex", text="ex")])
    detail_exception.contents = 123  # 非可迭代

    windows = [
        ("R", NS(uri="window://R/blob"), detail_blob),
        ("R", NS(uri="window://R/u"), detail_unknown),
        ("R", NS(uri="window://R/ex"), detail_exception),
    ]
    ret = await organize_desktop(windows=windows, size=None, history=tuple())
    # 断言：
    # 1) blob 项包含文本 t1（Blob 本身不拼接文本）
    assert any(x.startswith("window://R/blob") and "t1" in x for x in ret)
    # 2) unknown 类型不影响已有文本 t2
    assert any(x.startswith("window://R/u") and "t2" in x for x in ret)
    # 3) 渲染异常时仅返回 URI
    assert any(x == "window://R/ex" for x in ret)


@pytest.mark.asyncio
async def test_server_level_cap_breaks_iteration():
    """
    当第一个服务器已满足 size 上限时，后续服务器在服务器层级立即中断。
    When cap reached after first server, loop breaks before processing later servers.
    """
    from types import SimpleNamespace as NS

    from mcp.types import ReadResourceResult, TextResourceContents

    d_a = ReadResourceResult(contents=[TextResourceContents(uri="window://A/a", text="a")])
    d_b = ReadResourceResult(contents=[TextResourceContents(uri="window://B/b", text="b")])
    windows = [
        ("A", NS(uri="window://A/a"), d_a),
        ("B", NS(uri="window://B/b"), d_b),
    ]
    # 使用 history 让 A 在前，size=1 使得在进入 B 时触发服务器层级的 break
    ret = await organize_desktop(windows=windows, size=1, history=({"server": "A"},))
    assert len(ret) == 1 and ret[0].startswith("window://A/a")



# -*- coding: utf-8 -*-
# filename: test_sse_client.py
# @Time    : 2025/8/19 19:28
# @Author  : JQQ
# @Email   : jqq1716@gmail.com
# @Software: PyCharm
from collections.abc import Callable

import pytest
from mcp.client.session_group import SseServerParameters
from mcp.types import CallToolResult, Tool
from transitions import MachineError

from a2c_smcp.computer.mcp_clients.sse_client import SseMCPClient


@pytest.mark.asyncio
async def test_state_transitions(
    sse_server,
    sse_params: SseServerParameters,
    track_state: tuple[Callable[[str, str], None], list[tuple[str, str]]],
) -> None:
    """
    测试客户端状态转换
    Test client state transitions
    """
    callback, history = track_state
    client: SseMCPClient = SseMCPClient(sse_params, state_change_callback=callback)
    assert client.state == "initialized"
    await client.aconnect()
    assert client.state == "connected"
    assert ("initialized", "connected") in history
    await client.adisconnect()
    assert client.state == "disconnected"
    assert ("connected", "disconnected") in history
    await client.ainitialize()
    assert client.state == "initialized"
    assert ("disconnected", "initialized") in history


@pytest.mark.asyncio
async def test_list_tools(sse_server, sse_params: SseServerParameters) -> None:
    """
    测试获取工具列表功能
    Test list_tools functionality
    """
    client: SseMCPClient = SseMCPClient(sse_params)
    await client.aconnect()
    tools: list[Tool] = await client.list_tools()
    assert len(tools) == 2
    assert tools[0].name == "test_tool"
    assert tools[0].description == "A test tool"
    await client.adisconnect()


@pytest.mark.asyncio
async def test_call_tool_success(sse_server, sse_params: SseServerParameters) -> None:
    """
    测试成功调用工具
    Test successful tool call
    """
    client: SseMCPClient = SseMCPClient(sse_params)
    await client.aconnect()
    result: CallToolResult = await client.call_tool("test_tool", {})
    assert isinstance(result, CallToolResult)
    assert not result.isError
    assert result.content[0].text == "Called test_tool"
    await client.adisconnect()


@pytest.mark.asyncio
async def test_call_tool_failure(sse_server, sse_params: SseServerParameters) -> None:
    """
    测试工具调用失败场景
    Test tool call failure
    """
    client = SseMCPClient(sse_params)
    await client.aconnect()

    # 调用不存在的工具
    result = await client.call_tool("nonexistent_tool", {})
    assert result.isError, "调用不存在的工具应该失败"

    await client.adisconnect()


@pytest.mark.asyncio
async def test_async_session_property(sse_server, sse_params: SseServerParameters) -> None:
    """
    测试 async_session 属性
    Test async_session property
    """
    client = SseMCPClient(sse_params)
    # 未连接状态下会话为 None
    assert client._async_session is None

    # 访问 async_session 会自动连接
    session = await client.async_session
    assert session is not None

    await client.adisconnect()


@pytest.mark.asyncio
async def test_error_recovery(
    sse_server,
    sse_params: SseServerParameters,
    track_state: tuple[Callable[[str, str], None], list[tuple[str, str]]],
) -> None:
    """
    测试错误状态恢复
    Test error recovery
    """
    callback, history = track_state
    client = SseMCPClient(sse_params, state_change_callback=callback)
    await client.aconnect()

    # 强制进入错误状态
    await client.aerror()
    assert client.state == "error"
    assert any(from_state == "connected" and to_state == "error" for from_state, to_state in history)

    # 从错误状态恢复
    await client.ainitialize()
    assert client.state == "initialized"
    assert ("error", "initialized") in history

    # 尝试重新连接
    await client.aconnect()
    assert client.state == "connected"

    await client.adisconnect()


@pytest.mark.asyncio
async def test_invalid_state_operations(sse_server, sse_params: SseServerParameters) -> None:
    """
    测试在无效状态下执行操作
    Test invalid state operations
    """
    client: SseMCPClient = SseMCPClient(sse_params)
    with pytest.raises(ConnectionError):
        await client.call_tool("test_tool", {})
    with pytest.raises(ConnectionError):
        await client.list_tools()
    with pytest.raises(MachineError):
        await client.adisconnect()


@pytest.mark.asyncio
async def test_sse_message_handler_receives_list_changed_notifications(
    sse_server,
    sse_params: SseServerParameters,
) -> None:
    """
    中文: 验证在初始化 SseMCPClient 时传入 message_handler，能接收 listChanged 通知。
    英文: Verify that passing message_handler to SseMCPClient captures listChanged notifications.
    """
    received = {"tools": 0, "resources": 0, "prompts": 0}

    async def message_handler(message):
        from mcp.types import (
            PromptListChangedNotification,
            ResourceListChangedNotification,
            ServerNotification,
            ToolListChangedNotification,
        )

        if isinstance(message, ServerNotification):
            if isinstance(message.root, ToolListChangedNotification):
                received["tools"] += 1
            elif isinstance(message.root, ResourceListChangedNotification):
                received["resources"] += 1
            elif isinstance(message.root, PromptListChangedNotification):
                received["prompts"] += 1

    client = SseMCPClient(sse_params, message_handler=message_handler)
    await client.aconnect()

    # 触发通知 / trigger notifications via server tool
    result = await client.call_tool("trigger_list_changed", {})
    assert isinstance(result, CallToolResult) or hasattr(result, "content")

    import anyio

    for _ in range(20):  # wait up to ~2s
        if received["tools"] >= 1 and received["resources"] >= 1 and received["prompts"] >= 1:
            break
        await anyio.sleep(0.1)

    assert received["tools"] >= 1
    assert received["resources"] >= 1
    assert received["prompts"] >= 1

    await client.adisconnect()



# -*- coding: utf-8 -*-
# filename: base_client.py
# @Time    : 2025/8/18 10:57
# @Author  : JQQ
# @Email   : jqq1716@gmail.com
# @Software: PyCharm
import asyncio
from abc import ABC, abstractmethod
from collections.abc import Awaitable, Callable
from contextlib import AsyncExitStack
from enum import StrEnum
from typing import Generic, TypeVar, cast

from mcp import ClientSession, Tool
from mcp.client.session import MessageHandlerFnT
from mcp.types import AnyUrl, CallToolResult, InitializeResult, ReadResourceResult, Resource, TextResourceContents
from pydantic import BaseModel
from transitions.core import EventData
from transitions.extensions import AsyncMachine

from a2c_smcp.utils import WindowURI, is_window_uri
from a2c_smcp.utils.async_property import async_property
from a2c_smcp.utils.logger import logger

# 泛型参数，用于约束 MCP Server 参数类型
# Generic parameter for constraining MCP Server parameter types
ParamsT = TypeVar("ParamsT", bound=BaseModel)


class STATES(StrEnum):
    initialized = "initialized"
    connected = "connected"
    disconnected = "disconnected"
    error = "error"


TRANSITIONS = [
    {
        "trigger": "aconnect",
        "source": STATES.initialized,
        "dest": STATES.connected,
        "prepare": "aprepare_connect",
        "conditions": "acan_connect",
        "before": "abefore_connect",
        "after": "aafter_connect",
    },
    {
        "trigger": "adisconnect",
        "source": STATES.connected,
        "dest": STATES.disconnected,
        "prepare": "aprepare_disconnect",
        "conditions": "acan_disconnect",
        "before": "abefore_disconnect",
        "after": "aafter_disconnect",
    },
    {
        "trigger": "aerror",
        "source": "*",
        "dest": STATES.error,
        "prepare": "aprepare_error",
        "conditions": "acan_error",
        "before": "abefore_error",
        "after": "aafter_error",
    },
    {
        "trigger": "ainitialize",
        "source": "*",
        "dest": STATES.initialized,
        "prepare": "aprepare_initialize",
        "conditions": "acan_initialize",
        "before": "abefore_initialize",
        "after": "aafter_initialize",
    },
]


class A2CAsyncMachine(AsyncMachine):
    @staticmethod
    async def await_all(callables: list[Callable]) -> list:
        """
        Executes callables without parameters in parallel and collects their results.

        A2C协议中，需要在状态机的状态变化函数之间管理异步上下文，但由于原生实现 await_all 方法使用 asyncio.gather会导致上下文打开与关闭处于
            不同的async task中进而导致关闭异常。因此重写此实现，将await_all方法变为同步执行。以此实现上下文打开与关闭处于同一个async task中

        Args:
            callables (list): A list of callable functions

        Returns:
            list: A list of results. Using asyncio the list will be in the same order as the passed callables.
        """
        ret = []
        for c in callables:
            ret.append(await c())
        return ret


class BaseMCPClient(ABC, Generic[ParamsT]):
    def __init__(
        self,
        params: ParamsT,
        state_change_callback: Callable[[str, str], None | Awaitable[None]] | None = None,
        message_handler: MessageHandlerFnT | None = None,
    ) -> None:
        """
        基类初始化

        Attributes:
            params (ParamsT): MCP Server启动参数
            state_change_callback (Callable[[str, str], None | Awaitable[None]]): 状态变化回调，兼容同步与异步
            message_handler (Callable[..., Awaitable[None]] | None):
                自定义消息处理回调，符合 MCP ClientSession 的 message_handler 要求；若提供，则在构建 ClientSession 时传入。
                Custom message handler callback compatible with MCP ClientSession's message_handler; if provided,
                    it will be passed when creating the ClientSession.
        """
        self.params: ParamsT = params
        self._state_change_callback = state_change_callback
        # 私有属性：用于处理 ServerNotification（如 listChanged）的通用回调；在创建 ClientSession 时传入
        # Private attribute: general callback to handle ServerNotification (e.g., listChanged);
        # forwarded to ClientSession on creation
        self._message_handler = message_handler
        self._aexit_stack = AsyncExitStack()
        self._async_session: ClientSession | None = None
        self._session_keep_alive_task: asyncio.Task | None = None
        self._create_session_success_event = asyncio.Event()
        self._create_session_failure_event = asyncio.Event()
        self._async_session_closed_event = asyncio.Event()
        # 私有属性：初始化结果（用于后续能力/元信息使用）；断开连接时需清理
        # Private attribute: InitializeResult cached for later capabilities/meta usage; must be cleared on disconnect
        self._initialize_result: InitializeResult | None = None

        # 初始化异步状态机
        self.machine = A2CAsyncMachine(
            model=self,
            states=STATES,
            transitions=TRANSITIONS,
            initial=STATES.initialized,
            send_event=True,  # 传递事件对象给回调
            auto_transitions=False,  # 禁用自动生成的状态转移
            ignore_invalid_triggers=False,  # 忽略无效触发器
        )

    async def _trigger_state_change(self, event: EventData) -> None:
        """
        触发状态变化回调，兼容同步与异步

        Args:
            event (EventData): 事件对象
        """
        if not self._state_change_callback:
            return

        callback_result = self._state_change_callback(event.transition.source, event.transition.dest)
        # 处理异步回调
        if isinstance(callback_result, Awaitable):
            await callback_result

    @async_property
    async def async_session(self) -> ClientSession:
        """
        异步会话对象

        Returns:
            ClientSession: MCP 官方异步会话，用于触发 MCP Server 指令
        """
        if self._async_session is None:
            await self.aconnect()
        return cast(ClientSession, self._async_session)

    @property
    def initialize_result(self) -> InitializeResult | None:
        """
        初始化结果只读访问（可能为None，表示未初始化或已清理）
        Read-only access for InitializeResult (may be None if not initialized or already cleaned)
        """
        return self._initialize_result

    @abstractmethod
    async def _create_async_session(self) -> ClientSession:
        """
        创建异步会话对象。一般在此方法内对需要保持的上下文压栈管理

        Returns:
            ClientSession: MCP 官方异步会话，用于触发 MCP Server 指令
        """
        raise NotImplementedError

    async def _keep_alive_task(self) -> None:
        """
        async_session 保活，进而保证其它连接可以正常使用它。

        在MCP源码设计中，xxx_client与ClientSession均使用了anyio的task_group来管理子任务。但这带来一个维护问题，在Manager中需要管理多个Client，如果
          Client的AsyncSession是基于anyio.task_group打开，那么在关闭时，必须严格按照打开顺序关闭，否则会导致anyio报错。基于这个anyio特性，因为我需要让
          ClientSession在一个独立的Asyncio Task中运行，如此可以保证这个上下文的打开关联在这个内部Task中，从而可以实现自由关闭。在Manager中可
          以独立启停Client

        在这个实现中主要完成以下几个工作：

        1. 完成 self._async_session的创建
        2. 将需要持续保证的上下文压栈 self._aexit_stack
        3. 通过 asyncio.Event().wait() 来保证上下文的持续，同时通过响应 self._session_keep_alive_task.done() 来完成上下文的关闭
        4. 得到关闭信号后，对 self._aexit_stack 里的上下文进行关闭
        """
        logger.debug(f"Session keep-alive task: {asyncio.current_task().get_name()}")
        try:
            # 创建异步会话，同时完成上下文的压栈
            self._async_session = await self._create_async_session()
            # 通知创建成功
            self._create_session_success_event.set()
            # 持续等待关闭信息
            try:
                # 等待任务的cancel
                await asyncio.Event().wait()
            except asyncio.CancelledError:
                # 任务被取消，完成上下文
                logger.debug(f"Session keep-alive task cancelled: {asyncio.current_task().get_name()}")
        except Exception as e:
            logger.error(f"Session keep-alive task error: {asyncio.current_task().get_name()}: {e}")
            self._create_session_failure_event.set()
            await self.aerror()

        finally:
            # 关闭上下文
            await self._aexit_stack.aclose()
            # 清理session
            self._async_session = None
            # 清理初始化结果，确保会话真正关闭时协议初始化态一并清理
            # Cleanup InitializeResult to align with actual session teardown
            self._initialize_result = None
            self._async_session_closed_event.set()

    # region 状态转换回调函数基类实现
    async def aprepare_connect(self, event: EventData) -> None:
        """连接准备操作（可重写）"""
        logger.debug(f"Preparing connection with event: {event}\n\nserver params: {self.params}")

    async def acan_connect(self, event: EventData) -> bool:
        """连接条件检查（可重写）"""
        logger.debug(f"Checking connection conditions with event: {event}\n\nserver params: {self.params}")
        return True

    async def abefore_connect(self, event: EventData) -> None:
        """连接前操作（可重写）"""
        logger.debug(f"Before connection actions with event: {event}\n\nserver params: {self.params}")

    async def on_enter_connected(self, event: EventData) -> None:
        """进入已连接状态（可重写）"""
        logger.debug(f"Entering connected state with event: {event}\n\nserver params: {self.params}")
        self._session_keep_alive_task = asyncio.create_task(self._keep_alive_task())
        # 等待会话创建成功
        await self._create_session_success_event.wait()
        # 初始化client_session
        # 存储初始化返回结果，供后续使用
        # Store InitializeResult for later use
        self._initialize_result = await (await self.async_session).initialize()

    async def aafter_connect(self, event: EventData) -> None:
        """连接后操作（可重写）"""
        logger.debug(f"After connection actions with event: {event}\n\nserver params: {self.params}")
        await self._trigger_state_change(event)

    async def aprepare_disconnect(self, event: EventData) -> None:
        """断开准备操作（可重写）"""
        logger.debug(f"Preparing disconnection with event: {event}\n\nserver params: {self.params}")

    async def acan_disconnect(self, event: EventData) -> bool:
        """断开条件检查（可重写）"""
        logger.debug(f"Checking disconnection conditions with event: {event}\n\nserver params: {self.params}")
        return (await self.async_session) is not None

    async def abefore_disconnect(self, event: EventData) -> None:
        """断开前操作（可重写）"""
        logger.debug(f"Before disconnection actions with event: {event}\n\nserver params: {self.params}")

    async def on_enter_disconnected(self, event: EventData) -> None:
        """状态机进入断开状态时的回调（可重写）"""
        logger.debug(f"Entering disconnected state with event: {event}\n\nserver params: {self.params}")
        # 关闭异步会话，保证资源的正常释放
        logger.debug(f"Enter disconnected state async task: {asyncio.current_task().get_name()}")
        await self._close_task()
        # 等待会话关闭
        await self._async_session_closed_event.wait()

    async def aafter_disconnect(self, event: EventData) -> None:
        """断开后操作（可重写）"""
        logger.debug(f"After disconnection actions with event: {event}\n\nserver params: {self.params}")
        await self._trigger_state_change(event)

    async def aprepare_error(self, event: EventData) -> None:
        """错误准备操作（可重写）"""
        logger.debug(f"Preparing error with event: {event}\n\nserver params: {self.params}")

    async def acan_error(self, event: EventData) -> bool:
        """错误条件检查（可重写）"""
        logger.debug(f"Checking error conditions with event: {event}\n\nserver params: {self.params}")
        return True

    async def abefore_error(self, event: EventData) -> None:
        """错误前操作（可重写）"""
        logger.debug(f"Before error actions with event: {event}\n\nserver params: {self.params}")

    async def on_enter_error(self, event: EventData) -> None:
        """状态机进入错误状态时的回调（可重写）"""
        logger.debug(f"Entered error state with event: {event}\n\nserver params: {self.params}")
        # 将所有异步Event全部clear
        await self._close_task()

    async def aafter_error(self, event: EventData) -> None:
        """错误后操作（可重写）"""
        logger.debug(f"After error actions with event: {event}\n\nserver params: {self.params}")
        await self._trigger_state_change(event)

    async def aprepare_initialize(self, event: EventData) -> None:
        """初始化准备操作（可重写）"""
        logger.debug(f"Preparing initialization with event: {event}\n\nserver params: {self.params}")

    async def acan_initialize(self, event: EventData) -> bool:
        """初始化条件检查（可重写）"""
        logger.debug(f"Checking initialization conditions with event: {event}\n\nserver params: {self.params}")
        return True

    async def abefore_initialize(self, event: EventData) -> None:
        """初始化前操作（可重写）"""
        logger.debug(f"Before initialization actions with event: {event}\n\nserver params: {self.params}")

    async def on_enter_initialized(self, event: EventData) -> None:
        """状态机进入初始化状态时的回调（可重写）"""
        logger.debug(f"Entered initialized state with event: {event}\n\nserver params: {self.params}")
        # 将所有异步Event全部clear
        self._create_session_success_event.clear()
        self._create_session_failure_event.clear()
        self._async_session_closed_event.clear()
        await self._close_task()

    async def aafter_initialize(self, event: EventData) -> None:
        """初始化后操作（可重写）"""
        logger.debug(f"After initialization actions with event: {event}\n\nserver params: {self.params}")
        await self._trigger_state_change(event)

    async def list_tools(self) -> list[Tool]:
        """
        获取可用工具列表，MCP协议获取接口可分页，在此会尝试获取所有。对于大模型使用场景而言，需要一次性给出所有可用工具，没有必要分页，如果数据量过大，则属于设计问题。

        Returns:
            list[Tool]: 工具列表
        """
        if self.state != STATES.connected:
            raise ConnectionError("Not connected to server")
        tools: list[Tool] = []
        if self.initialize_result and self.initialize_result.capabilities.tools:
            asession = cast(ClientSession, await self.async_session)
            ret = await asession.list_tools()
            tools.extend(ret.tools)
            while ret.nextCursor:
                ret = await asession.list_tools(cursor=ret.nextCursor)
                tools.extend(ret.tools)
        return tools

    async def list_windows(self) -> list[Resource]:
        """
        列出当前MCP服务可用的窗口资源列表。需要注意MCP Server如果想开启桌面模式必须打开 resources/subscribe

        同时开发者需要注意维护好 window:// 状态

        Returns:
            list[Resource]: 当前可用的窗口类资源
        """
        if (
            self.initialize_result
            and self.initialize_result.capabilities.resources
            and self.initialize_result.capabilities.resources.subscribe
        ):
            # Get available resources directly from client session
            try:
                asession = cast(ClientSession, await self.async_session)
                # 中文: 支持分页获取资源；与 list_tools 一致，穷举所有页后再进行过滤与订阅
                # 英文: Support pagination; same as list_tools, exhaust all pages then filter and subscribe
                resources: list[Resource] = []
                ret = await asession.list_resources()
                if ret:
                    resources.extend(ret.resources)
                    while ret.nextCursor:
                        ret = await asession.list_resources(cursor=ret.nextCursor)
                        resources.extend(ret.resources)
                # 返回满足WindowURI协议要求的Resource
                # Return only resources that conform to WindowURI (window:// scheme)
                filtered: list[tuple[Resource, int]] = []
                for res in resources:
                    # 类型守卫：快速判定并过滤非 window:// 资源
                    if not is_window_uri(res.uri):
                        continue
                    # 解析优先级（缺省为0）
                    uri = WindowURI(str(res.uri))
                    prio = uri.priority if uri.priority is not None else 0
                    filtered.append((res, prio))

                # 同一 MCP 内按 priority 降序排序（仅在本客户端内比较）
                filtered.sort(key=lambda x: x[1], reverse=True)
                # 如果当前MCP Server开启了 resources 的订阅模式，则将过滤出来的Resources进行订阅
                for r, _ in filtered:
                    await asession.subscribe_resource(r.uri)
                return [r for r, _ in filtered]
            except Exception as e:
                logger.error(f"Error listing resources for connector {self.params.model_dump(mode='json')}: {e}")
                return []
        else:
            return []

    async def get_window_detail(self, resource: Resource | str) -> ReadResourceResult:
        """
        中文: 读取单个窗口资源的详细内容（通过 MCP read_resource）。
        英文: Read details for a single window resource via MCP read_resource.

        Args:
            resource (Resource | str): 要读取的资源（或其 URI 字符串）。

        Returns:
            list[object]: 资源内容块列表（如 TextContent/BlobContent 等）。读取失败返回空列表。
        """
        try:
            asession = cast(ClientSession, await self.async_session)
            uri_val: AnyUrl
            if isinstance(resource, Resource):
                uri_val = resource.uri  # type: ignore[assignment]
            else:
                # 当传入为字符串时，交由底层进行校验/解析
                uri_val = AnyUrl(resource)  # type: ignore[call-arg]

            return await asession.read_resource(uri_val)
        except Exception as e:
            logger.error(f"Read window resource failed: {resource}: {e}")
            return ReadResourceResult(contents=[TextResourceContents(text="获取资源失败", uri=resource.uri)])

    async def call_tool(self, tool_name: str, params: dict) -> CallToolResult:
        """
        运行指定工具（子类必须实现）

        在此不需要再考虑工具Alias的问题，由外层Manager进行处理，因此直接尝试调用触发MCP协议即可

        Args:
            tool_name (str): 被调用的工具名称
            params (dict): 调用参数

        Returns:
            CallToolResult: 调用结果 MCP 协议标准制定
        """
        if self.state != STATES.connected:
            raise ConnectionError("Not connected to server")
        return await (await self.async_session).call_tool(tool_name, params)

    async def _close_task(self) -> None:
        """
        关闭异步任务
        """
        # 将所有异步Event全部clear
        if self._session_keep_alive_task and not self._session_keep_alive_task.done():
            self._session_keep_alive_task.cancel()

            # 等待_session_keep_alive_task结束
            try:
                await self._session_keep_alive_task
            except asyncio.CancelledError:
                logger.debug("Session keep-alive task was cancelled")
            except Exception as e:
                logger.error(f"Session keep-alive task failed: {e}")



# -*- coding: utf-8 -*-
"""
测试 a2c_smcp/server/auth.py
"""

from __future__ import annotations

from unittest.mock import AsyncMock

import pytest

from a2c_smcp.server.auth import DefaultAuthenticationProvider

# get_agent_id 方法已被移除，不再需要测试
# get_agent_id method has been removed, no longer need to test


@pytest.mark.asyncio
async def test_authenticate_admin_ok_and_missing_or_wrong_key():
    prov = DefaultAuthenticationProvider("admin_secret")
    sio = AsyncMock()
    environ = {}

    # 正确 key
    headers = [(b"x-api-key", b"admin_secret")]
    ok = await prov.authenticate(sio, environ, None, headers)
    assert ok is True

    # 缺失 key
    headers = []
    ok2 = await prov.authenticate(sio, environ, None, headers)
    assert ok2 is False

    # 错误 key
    headers = [(b"x-api-key", b"wrong")]
    ok3 = await prov.authenticate(sio, environ, None, headers)
    assert ok3 is False


# has_admin_permission 方法已被移除，管理员权限检查已集成到 authenticate 方法中
# has_admin_permission method has been removed, admin permission check is now integrated into authenticate method


@pytest.mark.asyncio
async def test_admin_permission_integrated_in_authenticate():
    """测试管理员权限检查已集成到认证方法中 / Test admin permission check integrated in authenticate method"""
    sio = AsyncMock()
    environ = {}

    # 无管理员密钥配置的提供者
    # Provider without admin secret configured
    prov1 = DefaultAuthenticationProvider(None)
    headers = [(b"x-api-key", b"any_key")]
    assert await prov1.authenticate(sio, environ, None, headers) is False

    # 有管理员密钥配置的提供者
    # Provider with admin secret configured
    prov2 = DefaultAuthenticationProvider("admin_secret")
    headers = [(b"x-api-key", b"admin_secret")]
    assert await prov2.authenticate(sio, environ, None, headers) is True



# -*- coding: utf-8 -*-
# 文件名: test_cached_values.py
# 作者: JQQ
# 创建日期: 2025/9/24
# 最后修改日期: 2025/9/24
# 版权: 2023 JQQ. All rights reserved.
# 依赖: pytest
# 描述:
#   中文: 覆盖 InputResolver 的当前值缓存增删改查能力。
#   English: Cover CRUD capabilities for current values cache in InputResolver.

import pytest

from a2c_smcp.computer.inputs.resolver import InputResolver
from a2c_smcp.computer.mcp_clients.model import MCPServerPromptStringInput


@pytest.mark.asyncio
async def test_inputresolver_cached_values_crud() -> None:
    # 中文: 准备一个包含一个输入定义的解析器
    # English: Prepare a resolver with a single input definition
    inputs = [MCPServerPromptStringInput(id="token", description="desc", default="D", password=False, type="promptString")]
    resolver = InputResolver(inputs, session=None)

    # 初始应无缓存 / no cache initially
    assert resolver.list_cached_values() == {}
    assert resolver.get_cached_value("token") is None

    # 设置缓存 / set cache
    ok = resolver.set_cached_value("token", "ABC")
    assert ok is True
    assert resolver.get_cached_value("token") == "ABC"

    # 列出缓存 / list cache
    snap = resolver.list_cached_values()
    assert snap == {"token": "ABC"}

    # 删除缓存 / delete cache
    removed = resolver.delete_cached_value("token")
    assert removed is True
    assert resolver.get_cached_value("token") is None

    # 清空缓存 / clear
    resolver.set_cached_value("token", "XYZ")
    resolver.clear_cache()
    assert resolver.list_cached_values() == {}

    # 删除不存在键返回 False / delete non-exist returns False
    assert resolver.delete_cached_value("nope") is False

    # 设置不存在 id 返回 False / set for unknown id returns False
    assert resolver.set_cached_value("nope", "x") is False



# -*- coding: utf-8 -*-
# filename: __init__.py
# @Time    : 2025/9/30 13:56
# @Author  : JQQ
# @Email   : jqq1716@gmail.com
# @Software: PyCharm

"""
A2C-SMCP Agent模块
A2C-SMCP Agent module

提供Agent端的SMCP协议客户端实现，包括同步和异步两种模式
Provides Agent-side SMCP protocol client implementation, including both synchronous and asynchronous modes
"""

from a2c_smcp.agent.auth import AgentAuthProvider, DefaultAgentAuthProvider
from a2c_smcp.agent.base import BaseAgentClient
from a2c_smcp.agent.client import AsyncSMCPAgentClient
from a2c_smcp.agent.sync_client import SMCPAgentClient
from a2c_smcp.agent.types import (
    AgentConfig,
    AgentEventHandler,
    AgentID,
    AgentIDGetter,
    AsyncAgentEventHandler,
    AsyncAgentIDGetter,
    AsyncToolCallCallback,
    ComputerID,
    RequestID,
    ToolCallCallback,
    ToolCallContext,
)

__all__ = [
    # 认证相关 / Authentication related
    "AgentAuthProvider",
    "DefaultAgentAuthProvider",
    # 客户端实现 / Client implementations
    "BaseAgentClient",
    "SMCPAgentClient",
    "AsyncSMCPAgentClient",
    # 类型定义 / Type definitions
    "AgentConfig",
    "AgentEventHandler",
    "AsyncAgentEventHandler",
    "AgentID",
    "ComputerID",
    "RequestID",
    "ToolCallContext",
    "ToolCallCallback",
    "AsyncToolCallCallback",
    "AgentIDGetter",
    "AsyncAgentIDGetter",
]



"""
* 文件名: namespace
* 作者: JQQ
* 创建日期: 2025/9/29
* 最后修改日期: 2025/9/29
* 版权: 2023 JQQ. All rights reserved.
* 依赖: socketio, loguru, pydantic
* 描述: SMCP协议Namespace实现 / SMCP protocol Namespace implementation
"""

import copy
from typing import cast

from pydantic import TypeAdapter

from a2c_smcp.server.auth import AuthenticationProvider
from a2c_smcp.server.base import BaseNamespace
from a2c_smcp.server.types import OFFICE_ID, SID
from a2c_smcp.server.utils import aget_all_sessions_in_office
from a2c_smcp.smcp import (
    CANCEL_TOOL_CALL_NOTIFICATION,
    ENTER_OFFICE_NOTIFICATION,
    GET_DESKTOP_EVENT,
    GET_TOOLS_EVENT,
    LEAVE_OFFICE_NOTIFICATION,
    SMCP_NAMESPACE,
    TOOL_CALL_EVENT,
    UPDATE_CONFIG_NOTIFICATION,
    UPDATE_DESKTOP_NOTIFICATION,
    UPDATE_TOOL_LIST_NOTIFICATION,
    AgentCallData,
    EnterOfficeNotification,
    EnterOfficeReq,
    GetDeskTopReq,
    GetDeskTopRet,
    GetToolsReq,
    GetToolsRet,
    LeaveOfficeNotification,
    LeaveOfficeReq,
    ListRoomReq,
    ListRoomRet,
    SessionInfo,
    ToolCallReq,
    UpdateComputerConfigReq,
    UpdateMCPConfigNotification,
)
from a2c_smcp.utils.logger import logger


class SMCPNamespace(BaseNamespace):
    """
    处理SMCP相关事件的Socket.IO命名空间
    Socket.IO namespace for handling SMCP-related events
    """

    def __init__(self, auth_provider: AuthenticationProvider) -> None:
        """
        初始化SMCP命名空间
        Initialize SMCP namespace

        Args:
            auth_provider (AuthenticationProvider): 认证提供者 / Authentication provider
        """
        super().__init__(namespace=SMCP_NAMESPACE, auth_provider=auth_provider)

    async def enter_room(self, sid: SID, room: OFFICE_ID, namespace: str | None = None) -> None:
        """
        客户端加入房间，相较于父方法，添加了对sid的合规校验。维护session中的sid和name字段。
        Client joins room, adds sid compliance validation compared to parent method.
        Maintains 'sid' and 'name' fields in session.

        Args:
            sid (SID): 客户端ID / Client ID
            room (OFFICE_ID): 房间ID / Room ID
            namespace (Optional[str]): 命名空间 / Namespace
        """
        session = await self.get_session(sid)

        # 确保session中有sid字段
        # Ensure 'sid' field exists in session
        if session.get("sid") != sid:
            session["sid"] = sid

        # 确保session中有name字段（如无可用默认值）
        # Ensure 'name' field exists in session (use default if missing)
        if not session.get("name"):
            session["name"] = f"{session.get('role', 'unknown')}_{sid[:6]}"

        if session["role"] == "agent":
            # 如果sid已经存在于某个房间中，并且房间号不是当前房间号
            # If sid already exists in a room and the room number is not the current room
            if session.get("office_id") and session.get("office_id") != room:
                logger.error(f"Agent sid: {sid} already in room: {session.get('office_id')}, can't join room: {room}")
                raise ValueError("Agent sid already in room")

            # 如果sid不存在于任何房间中
            # If sid doesn't exist in any room
            elif not session.get("office_id"):
                # 获取房间内所有参与者
                # Get all participants in the room
                participants = self.server.manager.get_participants(SMCP_NAMESPACE, room)

                # 检查房间内是否已有Agent
                # Check if there's already an Agent in the room
                for participant_sid, _participant_eio_sid in participants:
                    participant_session = await self.get_session(participant_sid)
                    if participant_session.get("role") == "agent":
                        raise ValueError("Agent already in room")
            else:
                logger.warning(f"Agent sid: {sid} already in room: {session.get('office_id')}. 正在重复加入房间")
                return
        else:
            # Computer可以切换房间，但需要注意向即将离开的房间广播离开消息
            # Computer can switch rooms, but need to broadcast leave message to the room being left
            if session.get("office_id") and (past_room := session.get("office_id")) != room:
                await self.leave_room(sid, past_room)
            elif session.get("office_id") == room:
                logger.warning(f"Computer sid: {sid} already in room: {session.get('office_id')}. 正在重复加入房间")
                return

        # 加入新房间
        # Join new room
        await super().enter_room(sid, room)

        # 保存sid与房间号的映射关系
        # Save mapping between sid and room number
        session["office_id"] = room
        await self.save_session(sid, session)

        # 注册name到sid的映射
        # Register name-to-sid mapping
        await self._register_name(session["name"], sid)

        # 根据角色发送不同的通知 / Send different notifications based on role
        notification_data: EnterOfficeNotification = {"office_id": room}
        if session.get("role") == "computer":
            notification_data["computer"] = session.get("name")
        else:
            notification_data["agent"] = session.get("name")

        # 广播加入新房间的消息至房间内其它人
        # Broadcast join message to others in the room
        await self.emit(
            ENTER_OFFICE_NOTIFICATION,
            notification_data,
            skip_sid=sid,
            room=room,
        )

    async def leave_room(self, sid: SID, room: OFFICE_ID, namespace: str | None = None) -> None:
        """
        在离开房间之前发布离开消息
        Publish leave message before leaving room

        Args:
            sid (SID): 客户端ID / Client ID
            room (OFFICE_ID): 房间ID / Room ID
            namespace (Optional[str]): 命名空间 / Namespace
        """
        session = await self.get_session(sid)

        # 构建离开通知，使用name而不是sid
        # Build leave notification using name instead of sid
        client_name = session.get("name", "")
        notification = (
            LeaveOfficeNotification(office_id=room, computer=client_name)
            if session.get("role") == "computer"
            else LeaveOfficeNotification(office_id=room, agent=client_name)
        )

        # 广播离开消息
        # Broadcast leave message
        await self.emit(LEAVE_OFFICE_NOTIFICATION, notification, skip_sid=sid, room=room)

        # 注销name映射
        # Unregister name mapping
        await self._unregister_name(sid)

        # 维护session中的office_id字段
        # Maintain office_id field in session
        if "office_id" in session:
            del session["office_id"]
        await self.save_session(sid, session)

        # 调用父类方法离开房间
        # Call parent method to leave room
        await super().leave_room(sid, room)

    async def on_server_join_office(self, sid: str, data: EnterOfficeReq) -> tuple[bool, str | None]:
        """
        事件名：server:join_office 由全局变量 JOIN_OFFICE_EVENT 定义
        Computer或者Agent加入房间，为了突显smcp的办公特性，因此加入房间的动作命名为join_office
        Event name: server:join_office defined by global variable JOIN_OFFICE_EVENT
        Computer or Agent joins room, named join_office to highlight SMCP office characteristics

        Args:
            sid (str): 客户端ID 可能是 Computer或者Agent / Client ID, could be Computer or Agent
            data (EnterOfficeReq): 加入房间的数据 / Room join data

        Returns:
            tuple[bool, Optional[str]]: 返回是否允许加入房间，以及可能的错误信息
                                      / Returns whether joining is allowed and possible error message
        """
        role_info = TypeAdapter(EnterOfficeReq).validate_python(data)
        expected_role = role_info["role"]

        session = await self.get_session(sid)
        backup_session = copy.deepcopy(session)

        try:
            # 检查角色是否匹配
            # Check if role matches
            if session.get("role") and session["role"] != expected_role:
                return False, f"Role mismatch, expected {expected_role}, but {session['role']} use this sid exists"

            # 设置会话信息
            # Set session information
            session["role"] = expected_role
            session["name"] = role_info["name"]
            await self.save_session(sid, session)

            # 加入房间
            # Join room
            await self.enter_room(sid, role_info["office_id"])
            return True, None

        except Exception as e:
            # 恢复会话状态
            # Restore session state
            await self.save_session(sid, backup_session)
            return False, f"Internal server error: {str(e)}"

    async def on_server_leave_office(self, sid: str, data: LeaveOfficeReq) -> tuple[bool, str | None]:
        """
        事件名：server:leave_office 由全局变量 LEAVE_OFFICE_EVENT 定义
        Computer或者Agent离开房间，为了突显smcp的办公特性，因此离开房间的动作命名为leave_office
        Event name: server:leave_office defined by global variable LEAVE_OFFICE_EVENT
        Computer or Agent leaves room, named leave_office to highlight SMCP office characteristics

        Args:
            sid (str): 客户端ID 可能是 Computer或者Agent / Client ID, could be Computer or Agent
            data (LeaveOfficeReq): 离开房间的数据 / Room leave data

        Returns:
            tuple[bool, Optional[str]]: 返回是否允许离开房间，以及可能的错误信息
                                      / Returns whether leaving is allowed and possible error message
        """
        try:
            await self.leave_room(sid, data["office_id"])
            return True, None
        except Exception as e:
            return False, f"Internal server error: {str(e)}"

    async def on_server_tool_call_cancel(self, sid: str, data: AgentCallData) -> None:
        """
        将事件广播至对应的房间内所有Computer，通知取消工具调用
        Broadcast event to all Computers in the corresponding room, notifying tool call cancellation

        Args:
            sid (str): 发起者ID，应该是Agent / Initiator ID, should be Agent
            data (AgentCallData): Agent调用数据 / Agent call data
        """
        session = await self.get_session(sid)
        assert session["role"] == "agent", "目前仅支持Agent调用取消ToolCall的操作"

        agent_call = TypeAdapter(AgentCallData).validate_python(data)
        assert session.get("name") == agent_call["agent"], "取消工具调用的广播仅可以由对应Agent发出"

        # 广播到 office 房间，而不是 Agent 的私有房间 / Broadcast to office room, not Agent's private room
        office_id = session.get("office_id")
        await self.emit(
            CANCEL_TOOL_CALL_NOTIFICATION,
            agent_call,
            room=office_id,
            skip_sid=sid,
        )

    async def on_server_update_config(self, sid: str, data: UpdateComputerConfigReq) -> None:
        """
        将事件广播至对应的房间内所有Computer，通知更新MCP配置
        Broadcast event to all Computers in the corresponding room, notifying MCP config update

        Args:
            sid (str): 发起者ID，应该是Computer / Initiator ID, should be Computer
            data (UpdateComputerConfigReq): 更新配置请求数据 / Update config request data
        """
        session = await self.get_session(sid)
        assert session["role"] == "computer", "目前仅支持Computer调用更新MCP配置的操作"

        update_config = TypeAdapter(UpdateComputerConfigReq).validate_python(data)

        await self.emit(
            UPDATE_CONFIG_NOTIFICATION,
            UpdateMCPConfigNotification(computer=update_config["computer"]),
            room=session["office_id"],
            skip_sid=sid,
        )

    async def on_server_update_tool_list(self, sid: str, data: UpdateComputerConfigReq) -> None:
        """
        将事件广播至对应的房间内其他参与者，通知工具列表更新。
        Broadcast to others in the room to notify tool list update.

        Args:
            sid (str): 发起者ID，应为Computer / Initiator ID, should be Computer
            data (UpdateComputerConfigReq): 载荷复用 UpdateConfigReq，仅需 computer 标识 / Reuse UpdateConfigReq for payload
        """
        session = await self.get_session(sid)
        assert session["role"] == "computer", "目前仅支持Computer上报工具列表变更"

        update_req = TypeAdapter(UpdateComputerConfigReq).validate_python(data)

        await self.emit(
            UPDATE_TOOL_LIST_NOTIFICATION,
            {"computer": update_req["computer"]},
            room=session.get("office_id"),
            skip_sid=sid,
        )

    async def on_client_tool_call(self, sid: str, data: ToolCallReq) -> dict:
        """
        响应工具调用。注意因为Namespace的方法名与事件名称有耦合，因此需要保证 TOOL_CALL_EVENT 是 tool_call
        Respond to tool call. Note that due to coupling between Namespace method names and event names,
        TOOL_CALL_EVENT must be "tool_call"

        如果未来全局变量 TOOL_CALL_EVENT = "tool_call" 有修改，这里的方法名也需要修改
        If the global variable TOOL_CALL_EVENT = "tool_call" is modified in the future,
            the method name here also needs to be modified

        Args:
            sid (str): 客户端ID，一般是AgentID / Client ID, usually AgentID
            data (ToolCallReq): 工具调用数据 / Tool call data

        Returns:
            dict: 工具调用结果 / Tool call result
        """
        session = await self.get_session(sid)
        assert session["role"] == "agent", "目前仅支持Agent调用工具"

        tool_call = TypeAdapter(ToolCallReq).validate_python(data)

        # 通过name获取computer的sid
        # Get computer's sid by name
        computer_name = tool_call["computer"]
        computer_sid = await self.get_sid_by_name(computer_name)
        if not computer_sid:
            raise ValueError(f"Computer with name '{computer_name}' not found")

        return cast(
            dict,
            await self.call(
                TOOL_CALL_EVENT,
                tool_call,
                to=computer_sid,
                timeout=tool_call["timeout"],
            ),
        )

    async def on_client_get_tools(self, sid: str, data: GetToolsReq) -> GetToolsRet:
        """
        获取指定Computer的工具列表
        Get tool list of specified Computer

        Args:
            sid (str): 发起者的ID，一般是Agent / Initiator ID, usually Agent
            data (GetToolsReq): 其中包含computer字段，指向Computer的Name / Contains computer field pointing to Computer's Name

        Returns:
            GetToolsRet: Computer的工具列表 / Computer's tool list
        """
        computer_name = data["computer"]

        # 通过name获取computer的sid
        # Get computer's sid by name
        computer_sid = await self.get_sid_by_name(computer_name)
        if not computer_sid:
            raise ValueError(f"Computer with name '{computer_name}' not found")

        session = await self.get_session(computer_sid)
        assert session["role"] == "computer", "目前仅支持Computer获取工具列表"

        # 验证Agent是否有权限获取该Computer的工具列表
        # Verify if Agent has permission to get this Computer's tool list
        agent_session = await self.get_session(sid)
        computer_office_id = session.get("office_id")
        agent_office_id = agent_session.get("office_id")

        assert computer_office_id == agent_office_id, "目前仅支持Agent获取自己房间内Computer的工具列表"

        client_response = await self.call(
            GET_TOOLS_EVENT,
            data,
            to=computer_sid,
            namespace=SMCP_NAMESPACE,
        )

        return TypeAdapter(GetToolsRet).validate_python(client_response)

    async def on_client_get_desktop(self, sid: str, data: GetDeskTopReq) -> GetDeskTopRet:
        """
        获取指定Computer的桌面信息（窗口组织后的视图）。
        Get desktop view from specified Computer.

        要求：Agent 与 Computer 需在同一 office。
        """
        computer_name = data["computer"]

        # 通过name获取computer的sid
        # Get computer's sid by name
        computer_sid = await self.get_sid_by_name(computer_name)
        if not computer_sid:
            raise ValueError(f"Computer with name '{computer_name}' not found")

        session = await self.get_session(computer_sid)
        assert session["role"] == "computer", "目前仅支持获取Computer桌面"

        agent_session = await self.get_session(sid)
        computer_office_id = session.get("office_id")
        agent_office_id = agent_session.get("office_id")
        assert computer_office_id == agent_office_id, "目前仅支持Agent获取自己房间内Computer的桌面"

        client_response = await self.call(
            GET_DESKTOP_EVENT,
            data,
            to=computer_sid,
            namespace=SMCP_NAMESPACE,
        )
        return TypeAdapter(GetDeskTopRet).validate_python(client_response)

    async def on_server_update_desktop(self, sid: str, data: UpdateComputerConfigReq) -> None:
        """
        将事件广播至对应的房间内其他参与者，通知桌面刷新。
        Broadcast to others in the room to notify desktop update.

        Args:
            sid (str): 发起者ID，应为Computer / Initiator ID, should be Computer
            data (UpdateComputerConfigReq): 载荷复用 UpdateConfigReq，仅需 computer 标识
        """
        session = await self.get_session(sid)
        assert session["role"] == "computer", "目前仅支持Computer上报桌面刷新"

        update_req = TypeAdapter(UpdateComputerConfigReq).validate_python(data)
        await self.emit(
            UPDATE_DESKTOP_NOTIFICATION,
            {"computer": update_req["computer"]},
            room=session.get("office_id"),
            skip_sid=sid,
        )

    async def on_server_list_room(self, sid: str, data: ListRoomReq) -> ListRoomRet:
        """
        列出指定房间内的所有会话信息。Agent可以通过此事件查询房间内的所有Computer和Agent。
        List all sessions in the specified room. Agent can query all Computers and Agents in the room via this event.

        Args:
            sid (str): 发起者ID，一般是Agent / Initiator ID, usually Agent
            data (ListRoomReq): 列出房间请求数据，包含office_id和req_id / List room request data with office_id and req_id

        Returns:
            ListRoomRet: 房间内所有会话信息列表 / List of all session info in the room
        """
        # 验证请求数据 / Validate request data
        list_room_req = TypeAdapter(ListRoomReq).validate_python(data)
        office_id = list_room_req["office_id"]
        req_id = list_room_req["req_id"]

        # 验证发起者权限：确保Agent在请求的房间内 / Verify initiator permission: ensure Agent is in the requested room
        agent_session = await self.get_session(sid)
        agent_office_id = agent_session.get("office_id")

        assert agent_office_id == office_id, f"Agent只能查询自己所在房间的会话信息。Agent office: {agent_office_id}, requested: {office_id}"

        # 使用工具函数获取房间内所有会话信息 / Use utility function to get all session info in the room
        all_sessions = await aget_all_sessions_in_office(office_id, self.server)

        # 转换为SessionInfo格式 / Convert to SessionInfo format
        sessions: list[SessionInfo] = []
        for session in all_sessions:
            if session.get("role") in ["computer", "agent"]:
                session_info: SessionInfo = {
                    "sid": session.get("sid", ""),
                    "name": session.get("name", ""),
                    "role": session["role"],
                    "office_id": session.get("office_id", ""),
                }
                sessions.append(session_info)

        return ListRoomRet(sessions=sessions, req_id=req_id)



"""
测试 ToolMeta tags 字段在配置解析时不会丢失
Test that ToolMeta tags field is not lost during config parsing
"""

import json

from pydantic import TypeAdapter

from a2c_smcp.smcp import MCPServerConfig as SMCPServerConfigDict


def test_tool_meta_tags_preserved_in_config_parsing():
    """
    测试通过 TypeAdapter 验证配置时，tags 字段不会丢失
    Test that tags field is preserved when validating config with TypeAdapter
    """
    # 模拟用户通过 CLI 传入的配置
    config_dict = {
        "name": "playwright",
        "type": "stdio",
        "disabled": False,
        "forbidden_tools": [],
        "tool_meta": {},
        "default_tool_meta": {"tags": ["browser"], "auto_apply": True},
        "server_parameters": {
            "command": "npx",
            "args": ["@playwright/mcp@latest"],
            "env": None,
            "cwd": None,
            "encoding": "utf-8",
            "encoding_error_handler": "strict",
        },
        "vrl": None,
    }

    # 使用 TypeAdapter 验证配置（模拟 CLI 的行为）
    validated = TypeAdapter(SMCPServerConfigDict).validate_python(config_dict)

    print(f"\nvalidated config: {json.dumps(validated, indent=2, ensure_ascii=False)}")
    print(f"\ndefault_tool_meta: {validated.get('default_tool_meta')}")

    # 验证 tags 字段没有丢失
    assert "default_tool_meta" in validated
    assert validated["default_tool_meta"] is not None
    assert "tags" in validated["default_tool_meta"]
    assert validated["default_tool_meta"]["tags"] == ["browser"]
    assert validated["default_tool_meta"]["auto_apply"] is True


def test_tool_meta_all_fields():
    """
    测试 ToolMeta 的所有字段都能正确解析
    Test that all ToolMeta fields can be correctly parsed
    """
    config_dict = {
        "name": "test_server",
        "type": "stdio",
        "disabled": False,
        "forbidden_tools": [],
        "tool_meta": {
            "test_tool": {
                "auto_apply": False,
                "alias": "custom_tool",
                "tags": ["tag1", "tag2"],
                "ret_object_mapper": {"key": "value"},
            },
        },
        "default_tool_meta": {
            "auto_apply": True,
            "alias": None,
            "tags": ["default_tag"],
            "ret_object_mapper": None,
        },
        "server_parameters": {
            "command": "test",
            "args": [],
            "env": None,
            "cwd": None,
            "encoding": "utf-8",
            "encoding_error_handler": "strict",
        },
    }

    validated = TypeAdapter(SMCPServerConfigDict).validate_python(config_dict)

    # 验证 tool_meta
    assert "test_tool" in validated["tool_meta"]
    tool_meta = validated["tool_meta"]["test_tool"]
    assert tool_meta["auto_apply"] is False
    assert tool_meta["alias"] == "custom_tool"
    assert tool_meta["tags"] == ["tag1", "tag2"]
    assert tool_meta["ret_object_mapper"] == {"key": "value"}

    # 验证 default_tool_meta
    default_meta = validated["default_tool_meta"]
    assert default_meta["auto_apply"] is True
    assert default_meta["tags"] == ["default_tag"]



# -*- coding: utf-8 -*-
# filename: test_cli_io.py
# @Time    : 2025/9/21 17:09
# @Author  : JQQ
# @Email   : jiaqia@qknode.com
# @Software: PyCharm

import json

import pytest

from a2c_smcp.computer.inputs.cli_io import ainput_pick, ainput_prompt, arun_command


@pytest.mark.asyncio
async def test_ainput_prompt_returns_default_on_interrupt(monkeypatch):
    class DummySession:
        async def prompt_async(self, *args, **kwargs):
            raise EOFError()

    # patch PromptSession constructor to return dummy
    import a2c_smcp.computer.inputs.cli_io as cli_io

    monkeypatch.setattr(cli_io, "PromptSession", lambda: DummySession())

    value = await ainput_prompt("Enter:", default="def", password=True)
    assert value == "def"


@pytest.mark.asyncio
async def test_ainput_prompt_uses_default_on_empty(monkeypatch):
    class DummySession:
        async def prompt_async(self, *args, **kwargs):
            return ""

    import a2c_smcp.computer.inputs.cli_io as cli_io

    monkeypatch.setattr(cli_io, "PromptSession", lambda: DummySession())

    value = await ainput_prompt("Enter:", default="hello")
    assert value == "hello"


@pytest.mark.asyncio
async def test_ainput_prompt_returns_value(monkeypatch):
    class DummySession:
        async def prompt_async(self, *args, **kwargs):
            return "world"

    import a2c_smcp.computer.inputs.cli_io as cli_io

    monkeypatch.setattr(cli_io, "PromptSession", lambda: DummySession())

    value = await ainput_prompt("Enter:")
    assert value == "world"


@pytest.mark.asyncio
async def test_ainput_pick_single_happy_path(monkeypatch):
    inputs = iter(["1"])  # pick index 1

    class DummySession:
        async def prompt_async(self, *args, **kwargs):
            return next(inputs)

    import a2c_smcp.computer.inputs.cli_io as cli_io

    # silence console printing by replacing console.print with no-op
    monkeypatch.setattr(cli_io, "PromptSession", lambda: DummySession())
    monkeypatch.setattr(cli_io.console_util.console, "print", lambda *a, **k: None)

    picked = await ainput_pick("Pick one", ["a", "b", "c"], multi=False)
    assert picked == "b"


@pytest.mark.asyncio
async def test_ainput_pick_multi_with_dedup_and_order(monkeypatch):
    inputs = iter(["2,0,2,0"])  # picks ["c","a","c","a"] -> dedup to ["c","a"]

    class DummySession:
        async def prompt_async(self, *args, **kwargs):
            return next(inputs)

    import a2c_smcp.computer.inputs.cli_io as cli_io

    monkeypatch.setattr(cli_io, "PromptSession", lambda: DummySession())
    monkeypatch.setattr(cli_io.console_util.console, "print", lambda *a, **k: None)

    picked = await ainput_pick("Pick multi", ["a", "b", "c"], multi=True)
    assert picked == ["c", "a"]


@pytest.mark.asyncio
async def test_ainput_pick_default_on_interrupt_and_empty(monkeypatch):
    # first raise KeyboardInterrupt then return empty; both should yield default
    calls = {"n": 0}

    class DummySession:
        async def prompt_async(self, *args, **kwargs):
            calls["n"] += 1
            if calls["n"] == 1:
                raise KeyboardInterrupt()
            return ""

    import a2c_smcp.computer.inputs.cli_io as cli_io

    monkeypatch.setattr(cli_io, "PromptSession", lambda: DummySession())
    monkeypatch.setattr(cli_io.console_util.console, "print", lambda *a, **k: None)

    picked = await ainput_pick("Pick one", ["a", "b"], default_index=1, multi=False)
    assert picked == "b"

    # empty input path
    calls["n"] = 0
    picked2 = await ainput_pick("Pick one", ["a", "b"], default_index=0, multi=True)
    assert picked2 == ["a"]


@pytest.mark.asyncio
async def test_arun_command_raw_and_lines_and_json():
    # raw
    out = await arun_command("echo hello", parse="raw")
    # macOS echo appends newline; function strips
    assert out == "hello"

    # lines
    out_lines = await arun_command("printf 'a\\nb\\n'", parse="lines")
    assert out_lines == ["a", "b"]

    # json success
    js = {"x": 1, "y": [1, 2]}
    cmd = f"python - <<'PY'\nimport json,sys; print(json.dumps({json.dumps(js)}))\nPY"
    out_json = await arun_command(cmd, parse="json")
    assert out_json == js


@pytest.mark.asyncio
async def test_arun_command_timeout_and_error():
    # timeout
    with pytest.raises(TimeoutError):
        await arun_command("sleep 2", timeout=0.1)

    # non-zero exit -> RuntimeError
    with pytest.raises(RuntimeError):
        await arun_command("bash -c 'exit 7'")


@pytest.mark.asyncio
async def test_ainput_prompt_with_provided_session_branches(monkeypatch):
    """覆盖 ainput_prompt 在提供 session 时的异常与返回分支。"""

    # 1) 提供 session 且抛出 KeyboardInterrupt -> 返回默认值
    class Sess1:
        async def prompt_async(self, *_, **__):
            raise KeyboardInterrupt()

    v1 = await ainput_prompt("Q:", default="D", session=Sess1())
    assert v1 == "D"

    # 2) 提供 session 返回非空值
    class Sess2:
        async def prompt_async(self, *_, **__):
            return "val"

    v2 = await ainput_prompt("Q:", session=Sess2())
    assert v2 == "val"


@pytest.mark.asyncio
async def test_ainput_pick_invalid_then_out_of_range_then_ok_single(monkeypatch):
    """单选：先触发 ValueError，再触发越界，再成功。"""
    # 输入顺序：无效 -> 越界 -> 正确
    inputs = iter(["x", "9", "1"])  # x 触发 ValueError；9 越界；1 有效

    class DummySession:
        async def prompt_async(self, *args, **kwargs):
            return next(inputs)

    import a2c_smcp.computer.inputs.cli_io as cli_io

    monkeypatch.setattr(cli_io, "PromptSession", lambda: DummySession())
    monkeypatch.setattr(cli_io.console_util.console, "print", lambda *a, **k: None)

    picked = await ainput_pick("Pick one", ["a", "b", "c"], multi=False)
    assert picked == "b"


@pytest.mark.asyncio
async def test_ainput_pick_invalid_then_out_of_range_then_ok_multi(monkeypatch):
    """多选：先触发 ValueError，再触发越界，再成功。"""
    inputs = iter(["bad", "5", "0,2"])  # bad -> ValueError；5 越界；0,2 有效

    class DummySession:
        async def prompt_async(self, *args, **kwargs):
            return next(inputs)

    import a2c_smcp.computer.inputs.cli_io as cli_io

    monkeypatch.setattr(cli_io, "PromptSession", lambda: DummySession())
    monkeypatch.setattr(cli_io.console_util.console, "print", lambda *a, **k: None)

    picked = await ainput_pick("Pick multi", ["a", "b", "c"], multi=True)
    assert picked == ["a", "c"]


@pytest.mark.asyncio
async def test_arun_command_json_fallback_and_exec_mode(tmp_path):
    """覆盖 json 解析失败的回退与 shell=False 执行分支。"""
    # 1) json 解析失败 -> 返回原始字符串
    out = await arun_command("echo not-json", parse="json")
    assert out == "not-json"

    # 2) shell=False 使用可执行路径与 cwd（实现约束：不接受参数，command 仅作可执行名）
    subdir = tmp_path / "d"
    subdir.mkdir()
    # 使用 /bin/echo（无参数，输出仅为换行，strip 后为空字符串）
    res = await arun_command("/bin/echo", shell=False, cwd=str(subdir), parse="raw")
    assert res == ""


@pytest.mark.asyncio
async def test_ainput_pick_interrupt_with_invalid_default_multi(monkeypatch):
    """multi=True 且 default_index 越界时，KeyboardInterrupt 返回空列表。"""

    class DummySession:
        async def prompt_async(self, *args, **kwargs):
            raise KeyboardInterrupt()

    import a2c_smcp.computer.inputs.cli_io as cli_io

    monkeypatch.setattr(cli_io, "PromptSession", lambda: DummySession())
    monkeypatch.setattr(cli_io.console_util.console, "print", lambda *a, **k: None)

    picked = await ainput_pick("Pick multi", ["a", "b"], default_index=10, multi=True)
    assert picked == []


@pytest.mark.asyncio
async def test_ainput_pick_interrupt_with_invalid_default_single(monkeypatch):
    """multi=False 且 default_index 越界时，KeyboardInterrupt 返回空字符串。"""

    class DummySession:
        async def prompt_async(self, *args, **kwargs):
            raise EOFError()

    import a2c_smcp.computer.inputs.cli_io as cli_io

    monkeypatch.setattr(cli_io, "PromptSession", lambda: DummySession())
    monkeypatch.setattr(cli_io.console_util.console, "print", lambda *a, **k: None)

    picked = await ainput_pick("Pick one", ["a", "b"], default_index=999, multi=False)
    assert picked == ""


@pytest.mark.asyncio
async def test_arun_command_shell_false_missing_exec():
    """shell=False 且可执行不存在时应抛出 FileNotFoundError。"""
    with pytest.raises(FileNotFoundError):
        await arun_command("/bin/does-not-exist-xyz", shell=False)
